@@ -1,7 +1,10 @@
 #include <Elementary.h>
+#ifdef HAVE_CONFIG_H
+# include "elementary_config.h"
+#endif
 #ifndef ELM_LIB_QUICKLAUNCH
 static void
-my_anchorview_bt(void *data, Evas_Object *obj, void *event_info)
+my_anchorview_bt(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *av = data;
    elm_anchorview_hover_end(av);
@@ -68,7 +71,7 @@ my_anchorview_anchor(void *data, Evas_Object *obj, void *event_info)
 }
 
 void
-test_anchorview(void *data, Evas_Object *obj, void *event_info)
+test_anchorview(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *win, *bg, *av;
 
