@@ -1,4 +1,7 @@
 #include <Elementary.h>
+#ifdef HAVE_CONFIG_H
+# include "elementary_config.h"
+#endif
 #ifndef ELM_LIB_QUICKLAUNCH
 
 /* Hint:
@@ -18,14 +21,14 @@ typedef struct _Testitem
 
 
 static Elm_Genlist_Item_Class itc1;
-char *gl_label_get(const void *data, Evas_Object *obj, const char *part)
+char *gl_label_get(void *data, Evas_Object *obj __UNUSED__, const char *part __UNUSED__)
 {
    char buf[256];
-   snprintf(buf, sizeof(buf), "Item # %i", (int)data);
+   snprintf(buf, sizeof(buf), "Item # %i", (int)(long)data);
    return strdup(buf);
 }
 
-Evas_Object *gl_icon_get(const void *data, Evas_Object *obj, const char *part)
+Evas_Object *gl_icon_get(void *data __UNUSED__, Evas_Object *obj, const char *part __UNUSED__)
 {
    char buf[PATH_MAX];
    Evas_Object *ic = elm_icon_add(obj);
@@ -34,11 +37,11 @@ Evas_Object *gl_icon_get(const void *data, Evas_Object *obj, const char *part)
    evas_object_size_hint_aspect_set(ic, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
    return ic;
 }
-Eina_Bool gl_state_get(const void *data, Evas_Object *obj, const char *part)
+Eina_Bool gl_state_get(void *data __UNUSED__, Evas_Object *obj __UNUSED__, const char *part __UNUSED__)
 {
    return EINA_FALSE;
 }
-void gl_del(const void *data, Evas_Object *obj)
+void gl_del(void *data __UNUSED__, Evas_Object *obj __UNUSED__)
 {
 }
 
@@ -49,7 +52,7 @@ gl_sel(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-_move(void *data, Evas *evas, Evas_Object *obj, void *event_info)
+_move(void *data, Evas *evas __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Evas_Object *gl = data;
    Evas_Event_Mouse_Move *ev = event_info;
@@ -63,37 +66,37 @@ _move(void *data, Evas *evas, Evas_Object *obj, void *event_info)
 }
 
 static void
-_bt50_cb(void *data, Evas_Object *obj, void *event_info)
+_bt50_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
     elm_genlist_item_bring_in(data);
 }
 
 static void
-_bt1500_cb(void *data, Evas_Object *obj, void *event_info)
+_bt1500_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
     elm_genlist_item_middle_bring_in(data);
 }
 
 static void
-_gl_selected(void *data, Evas_Object *obj, void *event_info)
+_gl_selected(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    printf("selected: %p\n", event_info);
 }
 
 static void
-_gl_clicked(void *data, Evas_Object *obj, void *event_info)
+_gl_clicked(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    printf("clicked: %p\n", event_info);
 }
 
 static void
-_gl_longpress(void *data, Evas_Object *obj, void *event_info)
+_gl_longpress(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    printf("longpress %p\n", event_info);
 }
 
 void
-test_genlist(void *data, Evas_Object *obj, void *event_info)
+test_genlist(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *win, *bg, *gl, *bt_50, *bt_1500, *bx;
    Evas_Object *over;
@@ -153,11 +156,11 @@ test_genlist(void *data, Evas_Object *obj, void *event_info)
    for (i = 0; i < 2000; i++)
      {
         gli = elm_genlist_item_append(gl, &itc1,
-                                      (void *)i/* item data */,
+                                      (void *)(long)i/* item data */,
                                       NULL/* parent */,
                                       ELM_GENLIST_ITEM_NONE,
                                       gl_sel/* func */,
-                                      (void *)(i * 10)/* func data */);
+                                      (void *)(long)(i * 10)/* func data */);
         if (i == 50)
           evas_object_smart_callback_add(bt_50, "clicked", _bt50_cb, gli);
         else if (i == 1500)
@@ -170,17 +173,16 @@ test_genlist(void *data, Evas_Object *obj, void *event_info)
 /*************/
 
 static void
-my_gl_clear(void *data, Evas_Object *obj, void *event_info)
+my_gl_clear(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *gl = data;
    elm_genlist_clear(gl);
 }
 
 static void
-my_gl_add(void *data, Evas_Object *obj, void *event_info)
+my_gl_add(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *gl = data;
-   Elm_Genlist_Item *gli;
    static int i = 0;
 
    itc1.item_style     = "default";
@@ -189,20 +191,19 @@ my_gl_add(void *data, Evas_Object *obj, void *event_info)
    itc1.func.state_get = gl_state_get;
    itc1.func.del       = gl_del;
 
-   gli = elm_genlist_item_append(gl, &itc1,
-				 (void *)i/* item data */,
-				 NULL/* parent */,
-				 ELM_GENLIST_ITEM_NONE,
-				 gl_sel/* func */,
-				 (void *)(i * 10)/* func data */);
+   elm_genlist_item_append(gl, &itc1,
+                           (void *)(long)i/* item data */,
+                           NULL/* parent */,
+                           ELM_GENLIST_ITEM_NONE,
+                           gl_sel/* func */,
+                           (void *)(long)(i * 10)/* func data */);
    i++;
 }
 
 static void
-my_gl_insert_before(void *data, Evas_Object *obj, void *event_info)
+my_gl_insert_before(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *gl = data;
-   Elm_Genlist_Item *gli;
    static int i = 0;
    Elm_Genlist_Item *gli_selected;
 
@@ -219,20 +220,19 @@ my_gl_insert_before(void *data, Evas_Object *obj, void *event_info)
        return ;
    }
 
-   gli = elm_genlist_item_insert_before(gl, &itc1,
-				 (void *)i/* item data */,
-				 gli_selected /* item before */,
-				 ELM_GENLIST_ITEM_NONE,
-				 gl_sel/* func */,
-				 (void *)(i * 10)/* func data */);
+   elm_genlist_item_insert_before(gl, &itc1,
+                                  (void *)(long)i/* item data */,
+                                  gli_selected /* item before */,
+                                  ELM_GENLIST_ITEM_NONE,
+                                  gl_sel/* func */,
+                                  (void *)(long)(i * 10)/* func data */);
    i++;
 }
 
 static void
-my_gl_insert_after(void *data, Evas_Object *obj, void *event_info)
+my_gl_insert_after(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *gl = data;
-   Elm_Genlist_Item *gli;
    static int i = 0;
    Elm_Genlist_Item *gli_selected;
 
@@ -249,17 +249,17 @@ my_gl_insert_after(void *data, Evas_Object *obj, void *event_info)
        return ;
    }
 
-   gli = elm_genlist_item_insert_after(gl, &itc1,
-				 (void *)i/* item data */,
-				 gli_selected /* item after */,
-				 ELM_GENLIST_ITEM_NONE,
+   elm_genlist_item_insert_after(gl, &itc1,
+                                 (void *)(long)i/* item data */,
+                                 gli_selected /* item after */,
+                                 ELM_GENLIST_ITEM_NONE,
 				 gl_sel/* func */,
-				 (void *)(i * 10)/* func data */);
+				 (void *)(long)(i * 10)/* func data */);
    i++;
 }
 
 static void
-my_gl_del(void *data, Evas_Object *obj, void *event_info)
+my_gl_del(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *gl = data;
    Elm_Genlist_Item *gli = elm_genlist_selected_item_get(gl);
@@ -272,7 +272,7 @@ my_gl_del(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-my_gl_disable(void *data, Evas_Object *obj, void *event_info)
+my_gl_disable(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *gl = data;
    Elm_Genlist_Item *gli = elm_genlist_selected_item_get(gl);
@@ -287,7 +287,7 @@ my_gl_disable(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-my_gl_update_all(void *data, Evas_Object *obj, void *event_info)
+my_gl_update_all(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *gl = data;
    int i = 0;
@@ -302,7 +302,7 @@ my_gl_update_all(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-my_gl_first(void *data, Evas_Object *obj, void *event_info)
+my_gl_first(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *gl = data;
    Elm_Genlist_Item *gli = elm_genlist_first_item_get(gl);
@@ -312,7 +312,7 @@ my_gl_first(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-my_gl_last(void *data, Evas_Object *obj, void *event_info)
+my_gl_last(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *gl = data;
    Elm_Genlist_Item *gli = elm_genlist_last_item_get(gl);
@@ -322,20 +322,20 @@ my_gl_last(void *data, Evas_Object *obj, void *event_info)
 }
 
 static Eina_Bool
-my_gl_flush_delay(void *data)
+my_gl_flush_delay(void *data __UNUSED__)
 {
    elm_all_flush();
    return ECORE_CALLBACK_CANCEL;
 }
 
 static void
-my_gl_flush(void *data, Evas_Object *obj, void *event_info)
+my_gl_flush(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    ecore_timer_add(1.2, my_gl_flush_delay, NULL);
 }
 
 void
-test_genlist2(void *data, Evas_Object *obj, void *event_info)
+test_genlist2(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *win, *bg, *gl, *bx, *bx2, *bx3, *bt;
    Elm_Genlist_Item *gli[10];
@@ -507,21 +507,21 @@ test_genlist2(void *data, Evas_Object *obj, void *event_info)
 /*************/
 
 static Elm_Genlist_Item_Class itc2;
-char *gl2_label_get(const void *data, Evas_Object *obj, const char *part)
+char *gl2_label_get(void *data, Evas_Object *obj __UNUSED__, const char *part __UNUSED__)
 {
    const Testitem *tit = data;
    char buf[256];
    snprintf(buf, sizeof(buf), "Item mode %i", tit->mode);
    return strdup(buf);
 }
-Evas_Object *gl2_icon_get(const void *data, Evas_Object *obj, const char *part)
+Evas_Object *gl2_icon_get(void *data, Evas_Object *obj, const char *part)
 {
    const Testitem *tit = data;
    char buf[PATH_MAX];
    Evas_Object *ic = elm_icon_add(obj);
    if (!strcmp(part, "elm.swallow.icon"))
      {
-	if ((tit->mode & 0x3) == 0)
+	if (!(tit->mode & 0x3))
 	  snprintf(buf, sizeof(buf), "%s/images/logo_small.png", PACKAGE_DATA_DIR);
 	else if ((tit->mode & 0x3) == 1)
 	  snprintf(buf, sizeof(buf), "%s/images/logo.png", PACKAGE_DATA_DIR);
@@ -533,7 +533,7 @@ Evas_Object *gl2_icon_get(const void *data, Evas_Object *obj, const char *part)
      }
    else if (!strcmp(part, "elm.swallow.end"))
      {
-	if ((tit->mode & 0x3) == 0)
+	if (!(tit->mode & 0x3))
 	  snprintf(buf, sizeof(buf), "%s/images/sky_01.jpg", PACKAGE_DATA_DIR);
 	else if ((tit->mode & 0x3) == 1)
 	  snprintf(buf, sizeof(buf), "%s/images/sky_02.jpg", PACKAGE_DATA_DIR);
@@ -546,16 +546,16 @@ Evas_Object *gl2_icon_get(const void *data, Evas_Object *obj, const char *part)
    evas_object_size_hint_aspect_set(ic, EVAS_ASPECT_CONTROL_VERTICAL, 1, 1);
    return ic;
 }
-Eina_Bool gl2_state_get(const void *data, Evas_Object *obj, const char *part)
+Eina_Bool gl2_state_get(void *data __UNUSED__, Evas_Object *obj __UNUSED__, const char *part __UNUSED__)
 {
    return EINA_FALSE;
 }
-void gl2_del(const void *data, Evas_Object *obj)
+void gl2_del(void *data __UNUSED__, Evas_Object *obj __UNUSED__)
 {
 }
 
 static void
-my_gl_update(void *data, Evas_Object *obj, void *event_info)
+my_gl_update(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Testitem *tit = data;
    tit->mode++;
@@ -563,7 +563,7 @@ my_gl_update(void *data, Evas_Object *obj, void *event_info)
 }
 
 void
-test_genlist3(void *data, Evas_Object *obj, void *event_info)
+test_genlist3(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *win, *bg, *gl, *bx, *bx2, *bt;
    static Testitem tit[3];
@@ -649,7 +649,7 @@ test_genlist3(void *data, Evas_Object *obj, void *event_info)
 /*************/
 
 static void
-my_gl_item_check_changed(void *data, Evas_Object *obj, void *event_info)
+my_gl_item_check_changed(void *data, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Testitem *tit = data;
    tit->onoff = elm_check_state_get(obj);
@@ -657,17 +657,16 @@ my_gl_item_check_changed(void *data, Evas_Object *obj, void *event_info)
 }
 
 static Elm_Genlist_Item_Class itc3;
-char *gl3_label_get(const void *data, Evas_Object *obj, const char *part)
+char *gl3_label_get(void *data, Evas_Object *obj __UNUSED__, const char *part __UNUSED__)
 {
    const Testitem *tit = data;
    char buf[256];
    snprintf(buf, sizeof(buf), "Item mode %i", tit->mode);
    return strdup(buf);
 }
-Evas_Object *gl3_icon_get(const void *data, Evas_Object *obj, const char *part)
+Evas_Object *gl3_icon_get(void *data, Evas_Object *obj, const char *part)
 {
    const Testitem *tit = data;
-   char buf[PATH_MAX];
    if (!strcmp(part, "elm.swallow.icon"))
      {
         Evas_Object *lb;
@@ -678,23 +677,6 @@ Evas_Object *gl3_icon_get(const void *data, Evas_Object *obj, const char *part)
         elm_label_label_set(lb, "ashj ascjscjh n asjkl hcjlh ls hzshnn zjh sh zxjcjsnd h dfw sdv edev efe fwefvv vsd cvs ws wf  fvwf wd fwe f  we wef we wfe rfwewef wfv wswf wefg sdfws w wsdcfwcf wsc vdv  sdsd sdcd cv wsc sdcv wsc d sdcdcsd sdcdsc wdvd sdcsd wscxcv wssvd sd");
         evas_object_show(lb);
         return lb;
-        
-	Evas_Object *bx = elm_box_add(obj);
-	Evas_Object *ic;
-	ic = elm_icon_add(obj);
-	snprintf(buf, sizeof(buf), "%s/images/logo_small.png", PACKAGE_DATA_DIR);
-	elm_icon_file_set(ic, buf, NULL);
-	elm_icon_scale_set(ic, 0, 0);
-	evas_object_show(ic);
-	elm_box_pack_end(bx, ic);
-	ic = elm_icon_add(obj);
-	elm_icon_file_set(ic, buf, NULL);
-	elm_icon_scale_set(ic, 0, 0);
-	evas_object_show(ic);
-	elm_box_pack_end(bx, ic);
-        elm_box_horizontal_set(bx, 1);
-	evas_object_show(bx);
-	return bx;
      }
    else if (!strcmp(part, "elm.swallow.end"))
      {
@@ -708,16 +690,16 @@ Evas_Object *gl3_icon_get(const void *data, Evas_Object *obj, const char *part)
      }
    return NULL;
 }
-Eina_Bool gl3_state_get(const void *data, Evas_Object *obj, const char *part)
+Eina_Bool gl3_state_get(void *data __UNUSED__, Evas_Object *obj __UNUSED__, const char *part __UNUSED__)
 {
    return EINA_FALSE;
 }
-void gl3_del(const void *data, Evas_Object *obj)
+void gl3_del(void *data __UNUSED__, Evas_Object *obj __UNUSED__)
 {
 }
 
 void
-test_genlist4(void *data, Evas_Object *obj, void *event_info)
+test_genlist4(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *win, *bg, *gl, *bx, *bx2, *bt;
    static Testitem tit[3];
@@ -804,7 +786,7 @@ test_genlist4(void *data, Evas_Object *obj, void *event_info)
 
 /*************/
 static void
-my_gl_item_check_changed2(void *data, Evas_Object *obj, void *event_info)
+my_gl_item_check_changed2(void *data, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Testitem *tit = data;
    tit->onoff = elm_check_state_get(obj);
@@ -812,7 +794,7 @@ my_gl_item_check_changed2(void *data, Evas_Object *obj, void *event_info)
 }
 
 static Elm_Genlist_Item_Class itc5;
-char *gl5_label_get(const void *data, Evas_Object *obj, const char *part)
+char *gl5_label_get(void *data, Evas_Object *obj __UNUSED__, const char *part)
 {
    const Testitem *tit = data;
    char buf[256];
@@ -826,7 +808,7 @@ char *gl5_label_get(const void *data, Evas_Object *obj, const char *part)
      }
    return strdup(buf);
 }
-Evas_Object *gl5_icon_get(const void *data, Evas_Object *obj, const char *part)
+Evas_Object *gl5_icon_get(void *data, Evas_Object *obj, const char *part)
 {
    const Testitem *tit = data;
    char buf[PATH_MAX];
@@ -862,58 +844,82 @@ Evas_Object *gl5_icon_get(const void *data, Evas_Object *obj, const char *part)
      }
    return NULL;
 }
-Eina_Bool gl5_state_get(const void *data, Evas_Object *obj, const char *part)
+Eina_Bool gl5_state_get(void *data __UNUSED__, Evas_Object *obj __UNUSED__, const char *part __UNUSED__)
 {
    return EINA_FALSE;
 }
-void gl5_del(const void *data, Evas_Object *obj)
+void gl5_del(void *data __UNUSED__, Evas_Object *obj __UNUSED__)
 {
 }
 
 static void
-item_drag_up(void *data, Evas_Object *obj, void *event_info)
+item_drag_up(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    printf("drag up\n");
 }
 
 static void
-item_drag_down(void *data, Evas_Object *obj, void *event_info)
+item_drag_down(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    printf("drag down\n");
 }
 
 static void
-item_drag_left(void *data, Evas_Object *obj, void *event_info)
+item_drag_left(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    printf("drag left\n");
 }
 
 static void
-item_drag_right(void *data, Evas_Object *obj, void *event_info)
+item_drag_right(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    printf("drag right\n");
 }
 
 static void
-item_drag(void *data, Evas_Object *obj, void *event_info)
+scroll_top(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
+{
+   printf("Top edge!\n");
+}
+
+static void
+scroll_bottom(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
+{
+   printf("Bottom edge!\n");
+}
+
+static void
+scroll_left(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
+{
+   printf("Left edge!\n");
+}
+
+static void
+scroll_right(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
+{
+   printf("Right edge!\n");
+}
+
+static void
+item_drag(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    printf("drag\n");
 }
 
 static void
-item_drag_stop(void *data, Evas_Object *obj, void *event_info)
+item_drag_stop(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    printf("drag stop\n");
 }
 
 static void
-item_longpress(void *data, Evas_Object *obj, void *event_info)
+item_longpress(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    printf("longpress\n");
 }
 
 void
-test_genlist5(void *data, Evas_Object *obj, void *event_info)
+test_genlist5(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *win, *bg, *gl, *bx, *bx2, *bt;
    static Testitem tit[3];
@@ -963,6 +969,10 @@ test_genlist5(void *data, Evas_Object *obj, void *event_info)
    evas_object_smart_callback_add(gl, "drag,start,down", item_drag_down, NULL);
    evas_object_smart_callback_add(gl, "drag,start,left", item_drag_left, NULL);
    evas_object_smart_callback_add(gl, "drag,start,right", item_drag_right, NULL);
+   evas_object_smart_callback_add(gl, "scroll,edge,top", scroll_top, NULL);
+   evas_object_smart_callback_add(gl, "scroll,edge,bottom", scroll_bottom, NULL);
+   evas_object_smart_callback_add(gl, "scroll,edge,left", scroll_left, NULL);
+   evas_object_smart_callback_add(gl, "scroll,edge,right", scroll_right, NULL);
    evas_object_smart_callback_add(gl, "drag", item_drag, NULL);
    evas_object_smart_callback_add(gl, "drag,stop", item_drag_stop, NULL);
    evas_object_smart_callback_add(gl, "longpressed", item_longpress, NULL);
@@ -1009,53 +1019,62 @@ test_genlist5(void *data, Evas_Object *obj, void *event_info)
 static Elm_Genlist_Item_Class itc4;
 
 static void
-gl4_sel(void *data, Evas_Object *obj, void *event_info)
+gl4_sel(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
+   Elm_Genlist_Item *it = (Elm_Genlist_Item *)event_info;
+   int depth = 0;
+
+   depth = elm_genlist_item_expanded_depth_get(it);
+   printf("expanded depth for selected item is %d\n", depth);
+
 }
 static void
-gl4_exp(void *data, Evas_Object *obj, void *event_info)
+gl4_exp(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Elm_Genlist_Item *it = event_info;
    Evas_Object *gl = elm_genlist_item_genlist_get(it);
-   int val = (int)elm_genlist_item_data_get(it);
+   int val = (int)(long)elm_genlist_item_data_get(it);
    val *= 10;
    elm_genlist_item_append(gl, &itc4,
-			   (void *)(val + 1)/* item data */, it/* parent */, ELM_GENLIST_ITEM_NONE, gl4_sel/* func */,
+			   (void *)(long)(val + 1)/* item data */, it/* parent */,
+                           ELM_GENLIST_ITEM_NONE, gl4_sel/* func */,
 			   NULL/* func data */);
    elm_genlist_item_append(gl, &itc4,
-			   (void *)(val + 2)/* item data */, it/* parent */, ELM_GENLIST_ITEM_NONE, gl4_sel/* func */,
+			   (void *)(long)(val + 2)/* item data */, it/* parent */,
+                           ELM_GENLIST_ITEM_NONE, gl4_sel/* func */,
 			   NULL/* func data */);
    elm_genlist_item_append(gl, &itc4,
-			   (void *)(val + 3)/* item data */, it/* parent */, ELM_GENLIST_ITEM_SUBITEMS, gl4_sel/* func */,
+			   (void *)(long)(val + 3)/* item data */, it/* parent */,
+                           ELM_GENLIST_ITEM_SUBITEMS, gl4_sel/* func */,
 			   NULL/* func data */);
 }
 static void
-gl4_con(void *data, Evas_Object *obj, void *event_info)
+gl4_con(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Elm_Genlist_Item *it = event_info;
    elm_genlist_item_subitems_clear(it);
 }
 
 static void
-gl4_exp_req(void *data, Evas_Object *obj, void *event_info)
+gl4_exp_req(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Elm_Genlist_Item *it = event_info;
    elm_genlist_item_expanded_set(it, 1);
 }
 static void
-gl4_con_req(void *data, Evas_Object *obj, void *event_info)
+gl4_con_req(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Elm_Genlist_Item *it = event_info;
    elm_genlist_item_expanded_set(it, 0);
 }
 
-char *gl4_label_get(const void *data, Evas_Object *obj, const char *part)
+char *gl4_label_get(void *data, Evas_Object *obj __UNUSED__, const char *part __UNUSED__)
 {
    char buf[256];
-   snprintf(buf, sizeof(buf), "Item mode %i", (int)data);
+   snprintf(buf, sizeof(buf), "Item mode %i", (int)(long)data);
    return strdup(buf);
 }
-Evas_Object *gl4_icon_get(const void *data, Evas_Object *obj, const char *part)
+Evas_Object *gl4_icon_get(void *data __UNUSED__, Evas_Object *obj, const char *part)
 {
    char buf[PATH_MAX];
    if (!strcmp(part, "elm.swallow.icon"))
@@ -1076,16 +1095,16 @@ Evas_Object *gl4_icon_get(const void *data, Evas_Object *obj, const char *part)
      }
    return NULL;
 }
-Eina_Bool gl4_state_get(const void *data, Evas_Object *obj, const char *part)
+Eina_Bool gl4_state_get(void *data __UNUSED__, Evas_Object *obj __UNUSED__, const char *part __UNUSED__)
 {
    return EINA_FALSE;
 }
-void gl4_del(const void *data, Evas_Object *obj)
+void gl4_del(void *data __UNUSED__, Evas_Object *obj __UNUSED__)
 {
 }
 
 void
-test_genlist6(void *data, Evas_Object *obj, void *event_info)
+test_genlist6(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *win, *bg, *gl, *bx, *bx2, *bt;
 
@@ -1168,4 +1187,128 @@ test_genlist6(void *data, Evas_Object *obj, void *event_info)
    evas_object_resize(win, 320, 320);
    evas_object_show(win);
 }
+
+/*************/
+
+struct genlist7_data
+{
+  Evas_Object *win, *pager;
+};
+
+static Elm_Genlist_Item_Class itc7;
+static void
+gl_sel7(void *data, Evas_Object *obj, void *event_info)
+{
+   if (!event_info) return;
+   elm_genlist_item_item_class_update(event_info, &itc7);
+   printf("sel item data [%p] on genlist obj [%p], item pointer [%p], new item style [%s] \n", data, obj, event_info, itc7.item_style);
+}
+
+static void
+test_genlist7_back_cb(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
+{
+    struct genlist7_data *info = data;
+    if (!info) return;
+
+    elm_pager_content_pop(info->pager);
+}
+
+static void
+test_genlist7_swipe(void *data, Evas_Object *obj __UNUSED__, void *event_info)
+{
+   struct genlist7_data *info = data;
+   Evas_Object *box, *entry, *button;
+   char item_data[] = "Just a simple test";
+
+   if ((!event_info) || (!data)) return;
+
+   box = elm_box_add(info->win);
+   elm_box_homogenous_set(box, 0);
+   evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
+   evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
+   evas_object_show(box);
+
+   entry = elm_scrolled_entry_add(info->win);
+   elm_scrolled_entry_editable_set(entry, EINA_FALSE);
+   elm_scrolled_entry_entry_set(entry, item_data);
+   evas_object_size_hint_weight_set(entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
+   evas_object_size_hint_align_set(entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
+   evas_object_show(entry);
+
+   button = elm_button_add(info->win);
+   elm_button_label_set(button, "back");
+   evas_object_size_hint_weight_set(button, EVAS_HINT_EXPAND, 0);
+   evas_object_size_hint_align_set(button, EVAS_HINT_FILL, 0);
+   evas_object_smart_callback_add(button, "clicked", test_genlist7_back_cb,
+                                  info);
+   evas_object_show(button);
+
+   elm_box_pack_start(box, entry);
+   elm_box_pack_end(box, button);
+
+   elm_pager_content_push(info->pager, box);
+}
+
+void
+test_genlist7(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
+{
+   Evas_Object *win, *bg, *gl, *pager;
+   static struct genlist7_data info;
+   static Testitem tit[3];
+
+   win = elm_win_add(NULL, "genlist-7", ELM_WIN_BASIC);
+   elm_win_title_set(win, "Genlist 7");
+   elm_win_autodel_set(win, 1);
+   info.win = win;
+
+   bg = elm_bg_add(win);
+   elm_win_resize_object_add(win, bg);
+   evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
+   evas_object_show(bg);
+
+   pager = elm_pager_add(win);
+   elm_win_resize_object_add(win, pager);
+   evas_object_size_hint_weight_set(pager, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
+   evas_object_size_hint_align_set(pager, EVAS_HINT_FILL, EVAS_HINT_FILL);
+   evas_object_show(pager);
+   info.pager = pager;
+
+   gl = elm_genlist_add(win);
+   evas_object_size_hint_align_set(gl, EVAS_HINT_FILL, EVAS_HINT_FILL);
+   evas_object_size_hint_weight_set(gl, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
+   evas_object_smart_callback_add(gl, "swipe", test_genlist7_swipe, &info);
+   evas_object_show(gl);
+   elm_pager_content_push(pager, gl);
+
+   itc2.item_style     = "default";
+   itc2.func.label_get = gl2_label_get;
+   itc2.func.icon_get  = gl2_icon_get;
+   itc2.func.state_get = gl2_state_get;
+   itc2.func.del       = gl2_del;
+
+   itc7.item_style     = "double_label";
+   itc7.func.label_get = gl5_label_get;
+   itc7.func.icon_get  = gl5_icon_get;
+   itc7.func.state_get = gl5_state_get;
+   itc7.func.del       = gl5_del;
+
+   tit[0].mode = 0;
+   tit[0].item = elm_genlist_item_append(gl, &itc2,
+					 &(tit[0])/* item data */, NULL/* parent */,
+                                         ELM_GENLIST_ITEM_NONE, gl_sel7/* func */,
+					 NULL/* func data */);
+   tit[1].mode = 1;
+   tit[1].item = elm_genlist_item_append(gl, &itc2,
+					 &(tit[1])/* item data */, NULL/* parent */,
+                                         ELM_GENLIST_ITEM_NONE, gl_sel7/* func */,
+					 NULL/* func data */);
+   tit[2].mode = 2;
+   tit[2].item = elm_genlist_item_append(gl, &itc2,
+        				 &(tit[2])/* item data */, NULL/* parent */,
+                                         ELM_GENLIST_ITEM_NONE, gl_sel7/* func */,
+        				 NULL/* func data */);
+
+   evas_object_resize(win, 320, 320);
+   evas_object_show(win);
+}
 #endif
