@@ -1,13 +1,16 @@
 #include <Elementary.h>
+#ifdef HAVE_CONFIG_H
+# include "elementary_config.h"
+#endif
 #ifndef ELM_LIB_QUICKLAUNCH
 static void
-icon_clicked(void *data, Evas_Object *obj, void *event_info)
+icon_clicked(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    printf("clicked!\n");
 }
 
 void
-test_icon(void *data, Evas_Object *obj, void *event_info)
+test_icon(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *win, *ic;
    char buf[PATH_MAX];
