@@ -1,173 +1,137 @@
 #include <Elementary.h>
+#ifdef HAVE_CONFIG_H
+# include "elementary_config.h"
+#endif
 #ifndef ELM_LIB_QUICKLAUNCH
 
 static Evas_Object *rect;
 
 static void
-my_ph_clicked(void *data, Evas_Object *obj, void *event_info)
+my_ph_clicked(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("clicked\n");
 }
 
 static void
-my_ph_press(void *data, Evas_Object *obj, void *event_info)
+my_ph_press(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("press\n");
 }
 
 static void
-my_ph_longpressed(void *data, Evas_Object *obj, void *event_info)
+my_ph_longpressed(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("longpressed\n");
 }
 
 static void
-my_ph_clicked_double(void *data, Evas_Object *obj, void *event_info)
+my_ph_clicked_double(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("clicked,double\n");
 }
 
 static void
-my_ph_load(void *data, Evas_Object *obj, void *event_info)
+my_ph_load(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("load\n");
 }
 
 static void
-my_ph_loaded(void *data, Evas_Object *obj, void *event_info)
+my_ph_loaded(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("loaded\n");
 }
 
 static void
-my_ph_load_details(void *data, Evas_Object *obj, void *event_info)
+my_ph_load_details(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("load,details\n");
 }
 
 static void
-my_ph_loaded_details(void *data, Evas_Object *obj, void *event_info)
+my_ph_loaded_details(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("loaded,details\n");
 }
 
 static void
-my_ph_zoom_start(void *data, Evas_Object *obj, void *event_info)
+my_ph_zoom_start(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("zoom,start\n");
 }
 
 static void
-my_ph_zoom_stop(void *data, Evas_Object *obj, void *event_info)
+my_ph_zoom_stop(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("zoom,stop\n");
 }
 
 static void
-my_ph_zoom_change(void *data, Evas_Object *obj, void *event_info)
+my_ph_zoom_change(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("zoom,change\n");
 }
 
 static void
-my_ph_anim_start(void *data, Evas_Object *obj, void *event_info)
+my_ph_anim_start(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("anim,start\n");
 }
 
 static void
-my_ph_anim_stop(void *data, Evas_Object *obj, void *event_info)
+my_ph_anim_stop(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("anim,stop\n");
 }
 
 static void
-my_ph_drag_start(void *data, Evas_Object *obj, void *event_info)
+my_ph_drag_start(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("drag,start\n");
 }
 
 static void
-my_ph_drag_stop(void *data, Evas_Object *obj, void *event_info)
+my_ph_drag_stop(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    printf("drag_stop\n");
 }
 
 static void
-my_ph_scroll(void *data, Evas_Object *obj, void *event_info)
+my_ph_scroll(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
-//   Evas_Object *win = data;
    int x, y, w, h;
    elm_photocam_region_get(obj, &x, &y, &w, &h);
    printf("scroll %i %i %ix%i\n", x, y, w, h);
 }
 
 static void
-sel_done(void *data, Evas_Object *obj, void *event_info)
+my_bt_open(void *data, Evas_Object *obj __UNUSED__, void *event_info)
 {
-   Evas_Object *ph, *iw;
+   Evas_Object *ph = data;
+   const char *file = event_info;
 
-   ph = data;
-   iw = evas_object_data_get(ph, "inwin");
-   elm_photocam_file_set(ph, elm_fileselector_selected_get(obj));
-   evas_object_del(iw);
+   if (file)
+     elm_photocam_file_set(ph, file);
 }
 
 static void
-my_bt_open(void *data, Evas_Object *obj, void *event_info)
-{
-   Evas_Object *ph, *win;
-   Evas_Object *iw, *fs;
-   
-   ph = data;
-   win = evas_object_data_get(ph, "window");
-   iw = elm_win_inwin_add(win);
-   
-   fs = elm_fileselector_add(win);
-   elm_fileselector_expandable_set(fs, EINA_TRUE);
-   elm_fileselector_path_set(fs, getenv("HOME"));
-   evas_object_smart_callback_add(fs, "done", sel_done, ph);
-
-   evas_object_data_set(ph, "inwin", iw);
-   
-   elm_win_inwin_content_set(iw, fs);
-   evas_object_show(fs);
-   elm_win_inwin_activate(iw);
-}
-
-static void
-my_bt_show_reg(void *data, Evas_Object *obj, void *event_info)
+my_bt_show_reg(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    elm_photocam_image_region_show(data, 30, 50, 500, 300);
 }
 
 static void
-my_bt_bring_reg(void *data, Evas_Object *obj, void *event_info)
+my_bt_bring_reg(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 
 {
    elm_photocam_image_region_bring_in(data, 800, 300, 500, 300);
 }
 
 static void
-my_bt_zoom_in(void *data, Evas_Object *obj, void *event_info)
+my_bt_zoom_in(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    double zoom;
-   
+
    zoom = elm_photocam_zoom_get(data);
    zoom -= 0.5;
    elm_photocam_zoom_mode_set(data, ELM_PHOTOCAM_ZOOM_MODE_MANUAL);
@@ -175,10 +139,10 @@ my_bt_zoom_in(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-my_bt_zoom_out(void *data, Evas_Object *obj, void *event_info)
+my_bt_zoom_out(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    double zoom;
-   
+
    zoom = elm_photocam_zoom_get(data);
    zoom += 0.5;
    elm_photocam_zoom_mode_set(data, ELM_PHOTOCAM_ZOOM_MODE_MANUAL);
@@ -186,25 +150,25 @@ my_bt_zoom_out(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-my_bt_pause(void *data, Evas_Object *obj, void *event_info)
+my_bt_pause(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    elm_photocam_paused_set(data, !elm_photocam_paused_get(data));
 }
 
 static void
-my_bt_zoom_fit(void *data, Evas_Object *obj, void *event_info)
+my_bt_zoom_fit(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    elm_photocam_zoom_mode_set(data, ELM_PHOTOCAM_ZOOM_MODE_AUTO_FIT);
 }
 
 static void
-my_bt_zoom_fill(void *data, Evas_Object *obj, void *event_info)
+my_bt_zoom_fill(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    elm_photocam_zoom_mode_set(data, ELM_PHOTOCAM_ZOOM_MODE_AUTO_FILL);
 }
 
 static void
-_photocam_mouse_wheel_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_photocam_mouse_wheel_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Evas_Object *photocam = data;
    Evas_Event_Mouse_Wheel *ev = (Evas_Event_Mouse_Wheel*) event_info;
@@ -214,7 +178,7 @@ _photocam_mouse_wheel_cb(void *data, Evas *e, Evas_Object *obj, void *event_info
    ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
 
    zoom = elm_photocam_zoom_get(photocam);
-   if (ev->z>0 && zoom == 1) return;
+   if ((ev->z>0) && (zoom == 1)) return;
 
    if (ev->z > 0)
      zoom /= 2;
@@ -233,8 +197,8 @@ _photocam_mouse_wheel_cb(void *data, Evas *e, Evas_Object *obj, void *event_info
    if (zoom >= 1) elm_photocam_zoom_set(photocam, zoom);
 }
 
-   static void
-_photocam_move_resize_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
+static void
+_photocam_move_resize_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    int x,y,w,h;
 
@@ -244,7 +208,7 @@ _photocam_move_resize_cb(void *data, Evas *e, Evas_Object *obj, void *event_info
 }
 
 void
-test_photocam(void *data, Evas_Object *obj, void *event_info)
+test_photocam(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *win, *bg, *ph, *tb2, *bt;
    // these were just testing - use the "select photo" browser to select one
@@ -270,7 +234,7 @@ test_photocam(void *data, Evas_Object *obj, void *event_info)
    evas_object_size_hint_weight_set(ph, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_win_resize_object_add(win, ph);
    evas_object_data_set(ph, "window", win);
- 
+
    rect = evas_object_rectangle_add(evas_object_evas_get(win));
    evas_object_color_set(rect, 0, 0, 0, 0);
    evas_object_repeat_events_set(rect,1);
@@ -297,15 +261,15 @@ test_photocam(void *data, Evas_Object *obj, void *event_info)
    evas_object_smart_callback_add(ph, "scroll,drag,start", my_ph_drag_start, win);
    evas_object_smart_callback_add(ph, "scroll,drag,stop", my_ph_drag_stop, win);
    evas_object_smart_callback_add(ph, "scroll", my_ph_scroll, win);
-   
+
    elm_photocam_file_set(ph, img[1]);
-   
+
    evas_object_show(ph);
-   
+
    tb2 = elm_table_add(win);
    evas_object_size_hint_weight_set(tb2, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_win_resize_object_add(win, tb2);
-   
+
    bt = elm_button_add(win);
    elm_button_label_set(bt, "Z -");
    evas_object_smart_callback_add(bt, "clicked", my_bt_zoom_out, ph);
@@ -313,15 +277,15 @@ test_photocam(void *data, Evas_Object *obj, void *event_info)
    evas_object_size_hint_align_set(bt, 0.1, 0.1);
    elm_table_pack(tb2, bt, 0, 0, 1, 1);
    evas_object_show(bt);
-   
-   bt = elm_button_add(win);
-   elm_button_label_set(bt, "Select Photo");
-   evas_object_smart_callback_add(bt, "clicked", my_bt_open, ph);
+
+   bt = elm_fileselector_button_add(win);
+   elm_fileselector_button_label_set(bt, "Select Photo");
+   evas_object_smart_callback_add(bt, "file,chosen", my_bt_open, ph);
    evas_object_size_hint_weight_set(bt, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(bt, 0.5, 0.1);
    elm_table_pack(tb2, bt, 1, 0, 1, 1);
    evas_object_show(bt);
-   
+
    bt = elm_button_add(win);
    elm_button_label_set(bt, "Z +");
    evas_object_smart_callback_add(bt, "clicked", my_bt_zoom_in, ph);
@@ -330,7 +294,6 @@ test_photocam(void *data, Evas_Object *obj, void *event_info)
    elm_table_pack(tb2, bt, 2, 0, 1, 1);
    evas_object_show(bt);
 
-
    bt = elm_button_add(win);
    elm_button_label_set(bt, "Show 30,50 500x300");
    evas_object_smart_callback_add(bt, "clicked", my_bt_show_reg, ph);
@@ -338,7 +301,7 @@ test_photocam(void *data, Evas_Object *obj, void *event_info)
    evas_object_size_hint_align_set(bt, 0.1, 0.5);
    elm_table_pack(tb2, bt, 0, 1, 1, 1);
    evas_object_show(bt);
-   
+
    bt = elm_button_add(win);
    elm_button_label_set(bt, "Bring 800,300 500x300");
    evas_object_smart_callback_add(bt, "clicked", my_bt_bring_reg, ph);
@@ -346,8 +309,7 @@ test_photocam(void *data, Evas_Object *obj, void *event_info)
    evas_object_size_hint_align_set(bt, 0.9, 0.5);
    elm_table_pack(tb2, bt, 2, 1, 1, 1);
    evas_object_show(bt);
-   
-   
+
    bt = elm_button_add(win);
    elm_button_label_set(bt, "Pause On/Off");
    evas_object_smart_callback_add(bt, "clicked", my_bt_pause, ph);
@@ -355,7 +317,7 @@ test_photocam(void *data, Evas_Object *obj, void *event_info)
    evas_object_size_hint_align_set(bt, 0.1, 0.9);
    elm_table_pack(tb2, bt, 0, 2, 1, 1);
    evas_object_show(bt);
-   
+
    bt = elm_button_add(win);
    elm_button_label_set(bt, "Fit");
    evas_object_smart_callback_add(bt, "clicked", my_bt_zoom_fit, ph);
@@ -363,7 +325,7 @@ test_photocam(void *data, Evas_Object *obj, void *event_info)
    evas_object_size_hint_align_set(bt, 0.5, 0.9);
    elm_table_pack(tb2, bt, 1, 2, 1, 1);
    evas_object_show(bt);
-   
+
    bt = elm_button_add(win);
    elm_button_label_set(bt, "Fill");
    evas_object_smart_callback_add(bt, "clicked", my_bt_zoom_fill, ph);
@@ -371,9 +333,9 @@ test_photocam(void *data, Evas_Object *obj, void *event_info)
    evas_object_size_hint_align_set(bt, 0.9, 0.9);
    elm_table_pack(tb2, bt, 2, 2, 1, 1);
    evas_object_show(bt);
-   
+
    evas_object_show(tb2);
-   
+
    evas_object_resize(win, 800, 800);
    evas_object_show(win);
 }
