@@ -1,7 +1,10 @@
 #include <Elementary.h>
+#ifdef HAVE_CONFIG_H
+# include "elementary_config.h"
+#endif
 #ifndef ELM_LIB_QUICKLAUNCH
 void
-test_scaling(void *data, Evas_Object *obj, void *event_info)
+test_scaling(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *win, *bg, *bx, *bt;
 
@@ -60,7 +63,7 @@ test_scaling(void *data, Evas_Object *obj, void *event_info)
 }
 
 void
-test_scaling2(void *data, Evas_Object *obj, void *event_info)
+test_scaling2(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *win, *bg, *bx, *fr, *lb;
 
