@@ -14,9 +14,8 @@ typedef struct _Widget_Data Widget_Data;
 
 struct _Widget_Data
 {
-   Evas_Object *btn, *icon, *fs;
+   Evas_Object *self, *btn, *fs, *fsw;
    const char *window_title;
-   const char *btn_label;
    Evas_Coord w, h;
    struct {
       const char *path;
@@ -27,30 +26,23 @@ struct _Widget_Data
    Eina_Bool inwin_mode:1;
 };
 
-static const char DEFAULT_WINDOW_TITLE[] = "Select a file";
+#define DEFAULT_WINDOW_TITLE "Select a file"
 
 static const char *widtype = NULL;
+
 static void _del_hook(Evas_Object *obj);
 static void _theme_hook(Evas_Object *obj);
 static void _disable_hook(Evas_Object *obj);
 static void _sizing_eval(Evas_Object *obj);
 static void _changed_size_hints(void *data, Evas *e, Evas_Object *obj, void *event_info);
-static void _sub_del(void *data, Evas_Object *obj, void *event_info);
-static void _signal_clicked(void *data, Evas_Object *obj, const char *emission, const char *source);
-static void _signal_pressed(void *data, Evas_Object *obj, const char *emission, const char *source);
-static void _signal_unpressed(void *data, Evas_Object *obj, const char *emission, const char *source);
 static void _on_focus_hook(void *data, Evas_Object *obj);
 static void _selection_done(void *data, Evas_Object *obj, void *event_info);
+static void _activate(Widget_Data *wd);
 
-static const char SIG_CLICKED[] = "clicked";
-static const char SIG_UNPRESSED[] = "unpressed";
 static const char SIG_FILE_CHOSEN[] = "file,chosen";
-static const Evas_Smart_Cb_Description _signals[] = 
-{
-   {SIG_CLICKED, ""},
-   {SIG_UNPRESSED, ""},
-   {SIG_FILE_CHOSEN, "s"},
-   {NULL, NULL}
+static const Evas_Smart_Cb_Description _signals[] = {
+  {SIG_FILE_CHOSEN, "s"},
+  {NULL, NULL}
 };
 
 static void
@@ -62,7 +54,6 @@ _del_hook(Evas_Object *obj)
    wd = elm_widget_data_get(obj);
    if (!wd) return;
 
-   if (wd->btn_label) eina_stringshare_del(wd->btn_label);
    if (wd->window_title) eina_stringshare_del(wd->window_title);
    if (wd->fsd.path) eina_stringshare_del(wd->fsd.path);
    if (wd->fs)
@@ -79,37 +70,17 @@ _on_focus_hook(void *data __UNUSED__, Evas_Object *obj)
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
    if (elm_widget_focus_get(obj))
-     {
-	edje_object_signal_emit(wd->btn, "elm,action,focus", "elm");
-	evas_object_focus_set(wd->btn, 1);
-     }
-   else
-     {
-	edje_object_signal_emit(wd->btn, "elm,action,unfocus", "elm");
-	evas_object_focus_set(wd->btn, 0);
-     }
+     elm_widget_focus_steal(wd->btn);
 }
 
 static void
 _theme_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   char buf[4096];
    if (!wd) return;
-   _elm_theme_object_set(obj, wd->btn, "button", "base", elm_widget_style_get(obj));
-   if (wd->icon)
-     edje_object_part_swallow(wd->btn, "elm.swallow.content", wd->icon);
-   if (wd->btn_label)
-     edje_object_signal_emit(wd->btn, "elm,state,text,visible", "elm");
-   else
-     edje_object_signal_emit(wd->btn, "elm,state,text,hidden", "elm");
-   if (wd->icon)
-     edje_object_signal_emit(wd->btn, "elm,state,icon,visible", "elm");
-   else
-     edje_object_signal_emit(wd->btn, "elm,state,icon,hidden", "elm");
-   edje_object_part_text_set(wd->btn, "elm.text", wd->btn_label);
-   edje_object_message_signal_process(wd->btn);
-   edje_object_scale_set(wd->btn, 
-			 elm_widget_scale_get(obj) * _elm_config->scale);
+   snprintf(buf, sizeof(buf), "fileselector_button/%s", elm_widget_style_get(obj));
+   elm_object_style_set(wd->btn, buf);
    _sizing_eval(obj);
 }
 
@@ -118,10 +89,7 @@ _disable_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
-   if (elm_widget_disabled_get(obj))
-     edje_object_signal_emit(wd->btn, "elm,state,disabled", "elm");
-   else
-     edje_object_signal_emit(wd->btn, "elm,state,enabled", "elm");
+   elm_widget_disabled_set(wd->btn, elm_widget_disabled_get(obj));
 }
 
 static void
@@ -129,70 +97,43 @@ _sizing_eval(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord minw = -1, minh = -1, maxw = -1, maxh = -1;
-
    if (!wd) return;
-   elm_coords_finger_size_adjust(1, &minw, 1, &minh);
-   edje_object_size_min_restricted_calc(wd->btn, &minw, &minh, minw, minh);
-   elm_coords_finger_size_adjust(1, &minw, 1, &minh);
+   evas_object_size_hint_min_get(wd->btn, &minw, &minh);
+   evas_object_size_hint_max_get(wd->btn, &maxw, &maxh);
    evas_object_size_hint_min_set(obj, minw, minh);
    evas_object_size_hint_max_set(obj, maxw, maxh);
 }
 
 static void
-_changed_size_hints(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
+_changed_size_hints(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
    if (!wd) return;
-   if (obj != wd->icon) return;
    _sizing_eval(data);
 }
 
 static void
-_sub_del(void *data __UNUSED__, Evas_Object *obj, void *event_info)
+_activate_hook(Evas_Object *obj)
 {
-   Widget_Data *wd = elm_widget_data_get(obj);
-   Evas_Object *sub = event_info;
+   Widget_Data *wd;
+   wd = elm_widget_data_get(obj);
    if (!wd) return;
-   if (sub == wd->icon)
-     {
-	edje_object_signal_emit(wd->btn, "elm,state,icon,hidden", "elm");
-	evas_object_event_callback_del_full(sub,
-					    EVAS_CALLBACK_CHANGED_SIZE_HINTS,
-					    _changed_size_hints, obj);
-	wd->icon = NULL;
-	edje_object_message_signal_process(wd->btn);
-	_sizing_eval(obj);
-     }
+   _activate(wd);
 }
 
 static void
-_signal_clicked(void *data, Evas_Object *obj, const char *emission, const char *source)
+_button_clicked(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   Widget_Data *wd = elm_widget_data_get(data);
-   if (!wd) return;
-
-   evas_object_smart_callback_call(data, SIG_CLICKED, NULL);
-
-   /* safe guard when the theme does not emit the 'unpress' signal */
-   _signal_unpressed(data, obj, emission, source);
+   _activate(data);
 }
 
 static Evas_Object *
 _parent_win_get(Evas_Object *obj)
 {
-   Evas_Object *ret;
+   while ((obj) && (strcmp(elm_widget_type_get(obj), "win")))
+     obj = elm_object_parent_widget_get(obj);
 
-   ret = elm_object_parent_widget_get(obj);
-   if (!ret) return NULL;
-
-   if (strcmp(elm_widget_type_get(ret), "win") != 0)
-     {
-	DBG("Widget type (%s) is not \"win\", going up\n",
-	    elm_widget_type_get(ret));
-	return _parent_win_get(ret);
-     }
-
-   return ret;
+   return obj;
 }
 
 static Evas_Object *
@@ -214,44 +155,28 @@ _new_window_add(Widget_Data *wd)
 }
 
 static void
-_fs_launch(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
+_activate(Widget_Data *wd)
 {
-   Evas_Object *fs_btn, *win = NULL, *iw = NULL;
-   Eina_Bool win_fallback;
-   Widget_Data *wd;
+   Eina_Bool is_inwin = EINA_FALSE;
 
-   fs_btn = data;
-   wd = elm_widget_data_get(fs_btn);
-
-   if (!wd) return;
    if (wd->fs) return;
 
-   win_fallback = EINA_FALSE;
    if (wd->inwin_mode)
      {
-	win = _parent_win_get(fs_btn);
+	wd->fsw = _parent_win_get(wd->self);
 
-	if (!win)
-	  {
-	     ERR("No elementary window found as parent of the file selector "
-		 "button! Launching the file selector inside a new elementary"
-		 " window, then.");
-	     win_fallback = EINA_TRUE;
-	  }
+	if (!wd->fsw)
+          wd->fsw = _new_window_add(wd);
 	else
-	  {
-	     iw = elm_win_inwin_add(win);
-	     evas_object_data_set(fs_btn, "win", iw);
-	  }
+          {
+             wd->fsw = elm_win_inwin_add(wd->fsw);
+             is_inwin = EINA_TRUE;
+          }
      }
+   else
+     wd->fsw = _new_window_add(wd);
 
-   if ((!wd->inwin_mode) || (win_fallback))
-     {
-	win = _new_window_add(wd);
-	evas_object_data_set(fs_btn, "win", win);
-     }
-
-   wd->fs = elm_fileselector_add(win);
+   wd->fs = elm_fileselector_add(wd->fsw);
    elm_fileselector_expandable_set(wd->fs, wd->fsd.expandable);
    elm_fileselector_folder_only_set(wd->fs, wd->fsd.folder_only);
    elm_fileselector_is_save_set(wd->fs, wd->fsd.is_save);
@@ -259,56 +184,38 @@ _fs_launch(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSE
    evas_object_size_hint_weight_set(wd->fs, EVAS_HINT_EXPAND,
 				    EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(wd->fs, EVAS_HINT_FILL, EVAS_HINT_FILL);
-   evas_object_smart_callback_add(wd->fs, "done", _selection_done, fs_btn);
+   evas_object_smart_callback_add(wd->fs, "done", _selection_done, wd);
    evas_object_show(wd->fs);
 
-   if ((wd->inwin_mode) && (!win_fallback))
+   if (is_inwin)
      {
-	elm_win_inwin_content_set(iw, wd->fs);
-	elm_win_inwin_activate(iw);
+	elm_win_inwin_content_set(wd->fsw, wd->fs);
+	elm_win_inwin_activate(wd->fsw);
      }
    else
      {
-	elm_win_resize_object_add(win, wd->fs);
-	evas_object_show(win);
+	elm_win_resize_object_add(wd->fsw, wd->fs);
+	evas_object_show(wd->fsw);
      }
 }
 
 static void
 _selection_done(void *data, Evas_Object *obj __UNUSED__, void *event_info)
 {
-   Evas_Object *fs_btn, *win;
-   const char *file;
-   Widget_Data *wd;
-
-   file = event_info;
-   fs_btn = data;
-
-   wd = elm_widget_data_get(fs_btn);
+   const char *file = event_info;
+   Widget_Data *wd = data;
+   Evas_Object *del;
    if (!wd) return;
 
-   win = evas_object_data_del(fs_btn, "win");
-
-   evas_object_smart_callback_call(fs_btn, SIG_FILE_CHOSEN, event_info);
    if (file) eina_stringshare_replace(&wd->fsd.path, file);
 
+   del = wd->fsw;
    wd->fs = NULL;
-   evas_object_del(win);
-}
+   wd->fsw = NULL;
+   evas_object_del(del);
 
-static void
-_signal_pressed(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
-{
-   Widget_Data *wd = elm_widget_data_get(data);
-   if (!wd) return;
-}
-
-static void
-_signal_unpressed(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
-{
-   Widget_Data *wd = elm_widget_data_get(data);
-   if (!wd) return;
-   evas_object_smart_callback_call(data, SIG_UNPRESSED, NULL);
+   evas_object_smart_callback_call(wd->self, SIG_FILE_CHOSEN, 
+                                   (void *)wd->fsd.path);
 }
 
 /**
@@ -326,9 +233,13 @@ elm_fileselector_button_add(Evas_Object *parent)
    Evas *e;
    Widget_Data *wd;
 
+   EINA_SAFETY_ON_NULL_RETURN_VAL(parent, NULL);
+
    wd = ELM_NEW(Widget_Data);
    wd->window_title = eina_stringshare_add(DEFAULT_WINDOW_TITLE);
    wd->fsd.path = eina_stringshare_add(getenv("HOME"));
+   wd->fsd.expandable = _elm_config->fileselector_expand_enable;
+   wd->inwin_mode = _elm_config->inwin_dialogs_enable;
    wd->w = 400;
    wd->h = 400;
 
@@ -337,31 +248,23 @@ elm_fileselector_button_add(Evas_Object *parent)
    ELM_SET_WIDTYPE(widtype, "fileselector_button");
    elm_widget_type_set(obj, "fileselector_button");
    elm_widget_sub_object_add(parent, obj);
+   wd->self = obj;
    elm_widget_on_focus_hook_set(obj, _on_focus_hook, NULL);
    elm_widget_data_set(obj, wd);
    elm_widget_del_hook_set(obj, _del_hook);
    elm_widget_theme_hook_set(obj, _theme_hook);
    elm_widget_disable_hook_set(obj, _disable_hook);
-   elm_widget_can_focus_set(obj, 1);
+   elm_widget_can_focus_set(obj, EINA_TRUE);
+   elm_widget_activate_hook_set(obj, _activate_hook);
 
-   wd->btn = edje_object_add(e);
-   _elm_theme_object_set(obj, wd->btn, "button", "base", "default");
-   edje_object_signal_callback_add(wd->btn, "elm,action,click", "",
-                                   _signal_clicked, obj);
-   edje_object_signal_callback_add(wd->btn, "elm,action,click", "",
-                                   _fs_launch, obj);
-   edje_object_signal_callback_add(wd->btn, "elm,action,press", "",
-                                   _signal_pressed, obj);
-   edje_object_signal_callback_add(wd->btn, "elm,action,unpress", "",
-                                   _signal_unpressed, obj);
+   wd->btn = elm_button_add(parent);
    elm_widget_resize_object_set(obj, wd->btn);
+   evas_object_event_callback_add(wd->btn, EVAS_CALLBACK_CHANGED_SIZE_HINTS,
+				  _changed_size_hints, obj);
+   evas_object_smart_callback_add(wd->btn, "clicked", _button_clicked, wd);
+   elm_widget_sub_object_add(obj, wd->btn);
 
-   evas_object_smart_callback_add(obj, "sub-object-del", _sub_del, obj);
-
-   _sizing_eval(obj);
-
-   // TODO: convert Elementary to subclassing of Evas_Smart_Class
-   // TODO: and save some bytes, making descriptions per-class and not instance!
+   _theme_hook(obj);
    evas_object_smart_callbacks_descriptions_set(obj, _signals);
    return obj;
 }
@@ -380,61 +283,25 @@ elm_fileselector_button_label_set(Evas_Object *obj, const char *label)
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
-   eina_stringshare_replace(&wd->btn_label, label);
-   if (label)
-     edje_object_signal_emit(wd->btn, "elm,state,text,visible", "elm");
-   else
-     edje_object_signal_emit(wd->btn, "elm,state,text,hidden", "elm");
-   edje_object_message_signal_process(wd->btn);
-   edje_object_part_text_set(wd->btn, "elm.text", label);
-   _sizing_eval(obj);
+   elm_button_label_set(wd->btn, label);
 }
 
+
+/**
+ * Get the label used in the file selector button.
+ *
+ * @param obj The button object
+ * @return The button label
+ *
+ * @ingroup File_Selector_Button
+ */
 EAPI const char *
 elm_fileselector_button_label_get(const Evas_Object *obj)
 {
    ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return NULL;
-   return wd->btn_label;
-}
-
-/**
- * Set the path to start the button's file selector with, when clicked.
- *
- * @param obj The button object
- * @param path Path to a file/directory
- *
- * Default path is "HOME" environment variable's value.
- *
- * @ingroup File_Selector_Button
- */
-EAPI void
-elm_fileselector_button_selected_set(Evas_Object *obj, const char *path)
-{
-   ELM_CHECK_WIDTYPE(obj, widtype);
-   Widget_Data *wd = elm_widget_data_get(obj);
-   if (!wd) return;
-   eina_stringshare_replace(&wd->fsd.path, path);
-}
-
-/**
- * Get the <b>last</b> path which the button's file selector was set to.
- *
- * @param obj The button object
- * @param path Path to a file/directory
- *
- * Default path is "HOME" environment variable's value.
- *
- * @ingroup File_Selector_Button
- */
-EAPI const char *
-elm_fileselector_button_selected_get(const Evas_Object *obj)
-{
-   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
-   Widget_Data *wd = elm_widget_data_get(obj);
-   if (!wd) return NULL;
-   return wd->fsd.path;
+   return elm_button_label_get(wd->btn);
 }
 
 /**
@@ -453,14 +320,19 @@ elm_fileselector_button_window_title_set(Evas_Object *obj, const char *title)
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+
    if (!wd) return;
    eina_stringshare_replace(&wd->window_title, title);
+
+   if (wd->fsw)
+     elm_win_title_set(wd->fsw, wd->window_title);
 }
 
 /**
  * Get the title of the file selector button's window.
  *
  * @param obj The button object
+ * @return Title of the file selector button's window
  *
  * @ingroup File_Selector_Button
  */
@@ -469,6 +341,7 @@ elm_fileselector_button_window_title_get(const Evas_Object *obj)
 {
    ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
+
    if (!wd) return NULL;
    return wd->window_title;
 }
@@ -490,9 +363,13 @@ elm_fileselector_button_window_size_set(Evas_Object *obj, Evas_Coord width, Evas
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+
    if (!wd) return;
    wd->w = width;
    wd->h = height;
+
+   if (wd->fsw)
+     evas_object_resize(wd->fsw, wd->w, wd->h);
 }
 
 /**
@@ -509,6 +386,7 @@ elm_fileselector_button_window_size_get(const Evas_Object *obj, Evas_Coord *widt
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+
    if (!wd) return;
    if (width) *width = wd->w;
    if (height) *height = wd->h;
@@ -521,6 +399,7 @@ elm_fileselector_button_window_size_get(const Evas_Object *obj, Evas_Coord *widt
  * @param path The path string
  *
  * It must be a <b>directory</b> path.
+ * Default path is "HOME" environment variable's value.
  *
  * @ingroup File_Selector_Button
  */
@@ -529,8 +408,13 @@ elm_fileselector_button_path_set(Evas_Object *obj, const char *path)
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+
    if (!wd) return;
    eina_stringshare_replace(&wd->fsd.path, path);
+
+   if (wd->fs)
+     elm_fileselector_selected_set(wd->fs, wd->fsd.path);
+
 }
 
 /**
@@ -565,8 +449,12 @@ elm_fileselector_button_expandable_set(Evas_Object *obj, Eina_Bool value)
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+
    if (!wd) return;
    wd->fsd.expandable = value;
+
+   if (wd->fs)
+     elm_fileselector_expandable_set(wd->fs, wd->fsd.expandable);
 }
 
 /**
@@ -582,6 +470,7 @@ elm_fileselector_button_expandable_get(const Evas_Object *obj)
 {
    ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
    Widget_Data *wd = elm_widget_data_get(obj);
+
    if (!wd) return EINA_FALSE;
    return wd->fsd.expandable;
 }
@@ -600,8 +489,12 @@ elm_fileselector_button_folder_only_set(Evas_Object *obj, Eina_Bool value)
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+
    if (!wd) return;
    wd->fsd.folder_only = value;
+
+   if (wd->fs)
+     elm_fileselector_folder_only_set(wd->fs, wd->fsd.folder_only);
 }
 
 /**
@@ -617,6 +510,7 @@ elm_fileselector_button_folder_only_get(const Evas_Object *obj)
 {
    ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
    Widget_Data *wd = elm_widget_data_get(obj);
+
    if (!wd) return EINA_FALSE;
    return wd->fsd.folder_only;
 }
@@ -635,8 +529,12 @@ elm_fileselector_button_is_save_set(Evas_Object *obj, Eina_Bool value)
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+
    if (!wd) return;
    wd->fsd.is_save = value;
+
+   if (wd->fs)
+     elm_fileselector_is_save_set(wd->fs, wd->fsd.is_save);
 }
 
 /**
@@ -652,6 +550,7 @@ elm_fileselector_button_is_save_get(const Evas_Object *obj)
 {
    ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
    Widget_Data *wd = elm_widget_data_get(obj);
+
    if (!wd) return EINA_FALSE;
    return wd->fsd.is_save;
 }
@@ -671,6 +570,7 @@ elm_fileselector_button_inwin_mode_set(Evas_Object *obj, Eina_Bool value)
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+
    if (!wd) return;
    wd->inwin_mode = value;
 }
@@ -688,6 +588,7 @@ elm_fileselector_button_inwin_mode_get(const Evas_Object *obj)
 {
    ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
    Widget_Data *wd = elm_widget_data_get(obj);
+
    if (!wd) return EINA_FALSE;
    return wd->inwin_mode;
 }
@@ -696,9 +597,11 @@ elm_fileselector_button_inwin_mode_get(const Evas_Object *obj)
  * Set the icon used for the button
  *
  * Once the icon object is set, a previously set one will be deleted.
+ * If you want to keep that old content object, use the
+ * elm_fileselector_button_icon_unset() function.
  *
  * @param obj The button object
- * @param icon  The image for the button
+ * @param icon  The icon object for the button
  *
  * @ingroup File_Selector_Button
  */
@@ -707,27 +610,19 @@ elm_fileselector_button_icon_set(Evas_Object *obj, Evas_Object *icon)
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-   if (!wd) return;
-   if (wd->icon == icon) return;
-   if (wd->icon) evas_object_del(wd->icon);
-   wd->icon = icon;
-   if (icon)
+   if (!wd)
      {
-	elm_widget_sub_object_add(obj, icon);
-	evas_object_event_callback_add(icon, EVAS_CALLBACK_CHANGED_SIZE_HINTS,
-				       _changed_size_hints, obj);
-	edje_object_part_swallow(wd->btn, "elm.swallow.content", icon);
-	edje_object_signal_emit(wd->btn, "elm,state,icon,visible", "elm");
-	edje_object_message_signal_process(wd->btn);
+        evas_object_del(icon);
+        return;
      }
-   _sizing_eval(obj);
+   elm_button_icon_set(wd->btn, icon);
 }
 
 /**
  * Get the icon used for the button
  *
  * @param obj The button object
- * @return The image for the button
+ * @return The icon object that is being used
  *
  * @ingroup File_Selector_Button
  */
@@ -737,5 +632,24 @@ elm_fileselector_button_icon_get(const Evas_Object *obj)
    ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return NULL;
-   return wd->icon;
+   return elm_button_icon_get(wd->btn);
+}
+
+/**
+ * Unset the icon used for the button
+ *
+ * Unparent and return the icon object which was set for this widget.
+ *
+ * @param obj The button object
+ * @return The icon object that was being used
+ *
+ * @ingroup File_Selector_Button
+ */
+EAPI Evas_Object *
+elm_fileselector_button_icon_unset(Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
+   return elm_button_icon_unset(wd->btn);
 }
