@@ -14,12 +14,12 @@ typedef struct _Widget_Data Widget_Data;
 struct _Widget_Data
 {
    Evas_Object *clk;
-   double interval;
+   double interval, first_interval;
    Eina_Bool seconds : 1;
    Eina_Bool am_pm : 1;
    Eina_Bool edit : 1;
    Elm_Clock_Digedit digedit;
-   int hrs, min, sec;
+   int hrs, min, sec, timediff;
    Evas_Object *digit[6];
    Evas_Object *ampm;
    Evas_Object *sel_obj;
@@ -38,6 +38,7 @@ struct _Widget_Data
 static const char *widtype = NULL;
 static void _del_hook(Evas_Object *obj);
 static void _theme_hook(Evas_Object *obj);
+static void _on_focus_hook(void *data, Evas_Object *obj);
 static Eina_Bool _ticker(void *data);
 static Eina_Bool _signal_clock_val_up(void *data);
 static Eina_Bool _signal_clock_val_down(void *data);
@@ -78,9 +79,15 @@ _on_focus_hook(void *data __UNUSED__, Evas_Object *obj)
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
    if (elm_widget_focus_get(obj))
-     edje_object_signal_emit(wd->clk, "elm,action,focus", "elm");
+     {
+       edje_object_signal_emit(wd->clk, "elm,action,focus", "elm");
+       evas_object_focus_set(wd->clk, EINA_TRUE);
+     }
    else
-     edje_object_signal_emit(wd->clk, "elm,action,unfocus", "elm");
+     {
+       edje_object_signal_emit(wd->clk, "elm,action,unfocus", "elm");
+       evas_object_focus_set(wd->clk, EINA_FALSE);
+     }
 }
 
 static void
@@ -98,7 +105,7 @@ _signal_emit_hook(Evas_Object *obj, const char *emission, const char *source)
 }
 
 static void
-_signal_listen_hook(Evas_Object *obj, const char *emission, const char *source, void (*func_cb) (void *data, Evas_Object *o, const char *emission, const char *source), void *data)
+_signal_callback_add_hook(Evas_Object *obj, const char *emission, const char *source, void (*func_cb) (void *data, Evas_Object *o, const char *emission, const char *source), void *data)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    int i;
@@ -112,18 +119,32 @@ _signal_listen_hook(Evas_Object *obj, const char *emission, const char *source,
      }
 }
 
-static void *
-_signal_unlisten_hook(Evas_Object *obj, const char *emission, const char *source, void (*func_cb) (void *data, Evas_Object *o, const char *emission, const char *source))
+static void
+_signal_callback_del_hook(Evas_Object *obj, const char *emission, const char *source, void (*func_cb) (void *data, Evas_Object *o, const char *emission, const char *source), void *data)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    int i;
-   if (!wd) return NULL;
    for (i = 0; i < 6; i++)
      {
-	edje_object_signal_callback_del(wd->digit[i], emission, source,
-	      func_cb);
+	edje_object_signal_callback_del_full(wd->digit[i], emission, source,
+                                             func_cb, data);
      }
-   return edje_object_signal_callback_del(wd->clk, emission, source, func_cb);
+   edje_object_signal_callback_del_full(wd->clk, emission, source, func_cb,
+                                        data);
+}
+
+static void
+_timediff_set(Widget_Data *wd)
+{
+   struct timeval timev;
+   struct tm *tm;
+   time_t tt;
+   gettimeofday(&timev, NULL);
+   tt = (time_t)(timev.tv_sec);
+   tzset();
+   tm = localtime(&tt);
+   wd->timediff = (((wd->hrs - tm->tm_hour) * 60 +
+	    wd->min - tm->tm_min) * 60) + wd->sec - tm->tm_sec;
 }
 
 static Eina_Bool
@@ -140,7 +161,7 @@ _ticker(void *data)
    wd->ticker = ecore_timer_add(t, _ticker, data);
    if (!wd->edit)
      {
-	tt = (time_t)(timev.tv_sec);
+	tt = (time_t)(timev.tv_sec) + wd->timediff;
 	tzset();
 	tm = localtime(&tt);
 	if (tm)
@@ -158,7 +179,7 @@ static Eina_Bool
 _signal_clock_val_up(void *data)
 {
    Widget_Data *wd = elm_widget_data_get(data);
-   if (!wd) goto clock_val_up_cancel;
+   if (!wd) goto clock_val_up_exit_on_error;
    if (!wd->edit) goto clock_val_up_cancel;
    if (!wd->sel_obj) goto clock_val_up_cancel;
    if (wd->sel_obj == wd->digit[0])
@@ -203,6 +224,7 @@ _signal_clock_val_up(void *data)
    return ECORE_CALLBACK_RENEW;
 clock_val_up_cancel:
    wd->spin = NULL;
+clock_val_up_exit_on_error:
    return ECORE_CALLBACK_CANCEL;
 }
 
@@ -210,7 +232,7 @@ static Eina_Bool
 _signal_clock_val_down(void *data)
 {
    Widget_Data *wd = elm_widget_data_get(data);
-   if (!wd) goto clock_val_down_cancel;
+   if (!wd) goto clock_val_down_exit_on_error;
    if (!wd->edit) goto clock_val_down_cancel;
    if (!wd->sel_obj) goto clock_val_down_cancel;
    if (wd->sel_obj == wd->digit[0])
@@ -255,6 +277,7 @@ _signal_clock_val_down(void *data)
    return ECORE_CALLBACK_RENEW;
 clock_val_down_cancel:
    wd->spin = NULL;
+clock_val_down_exit_on_error:
    return ECORE_CALLBACK_CANCEL;
 }
 
@@ -263,7 +286,7 @@ _signal_clock_val_up_start(void *data, Evas_Object *obj, const char *emission __
 {
    Widget_Data *wd = elm_widget_data_get(data);
    if (!wd) return;
-   wd->interval = 0.85;
+   wd->interval = wd->first_interval;
    wd->sel_obj = obj;
    if (wd->spin) ecore_timer_del(wd->spin);
    wd->spin = ecore_timer_add(wd->interval, _signal_clock_val_up, data);
@@ -275,7 +298,7 @@ _signal_clock_val_down_start(void *data, Evas_Object *obj, const char *emission
 {
    Widget_Data *wd = elm_widget_data_get(data);
    if (!wd) return;
-   wd->interval = 0.85;
+   wd->interval = wd->first_interval;
    wd->sel_obj = obj;
    if (wd->spin) ecore_timer_del(wd->spin);
    wd->spin = ecore_timer_add(wd->interval, _signal_clock_val_down, data);
@@ -340,7 +363,7 @@ _time_update(Evas_Object *obj)
 	     _elm_theme_object_set(obj, wd->digit[i], "clock", "flipdigit", style);
 	     edje_object_scale_set(wd->digit[i], elm_widget_scale_get(obj) * 
                                    _elm_config->scale);
-	     if (wd->edit && (wd->digedit & (1 << i)))
+	     if ((wd->edit) && (wd->digedit & (1 << i)))
 	       edje_object_signal_emit(wd->digit[i], "elm,state,edit,on", "elm");
 	     edje_object_signal_callback_add(wd->digit[i], "elm,action,up,start",
 					     "", _signal_clock_val_up_start, obj);
@@ -409,7 +432,7 @@ _time_update(Evas_Object *obj)
 		  if (hrs > 12) hrs -= 12;
 		  ampm = 1;
 	       }
-	     else if (hrs == 0) hrs = 12;
+	     else if (!hrs) hrs = 12;
 	  }
 	d1 = hrs / 10;
 	d2 = hrs % 10;
@@ -507,6 +530,8 @@ elm_clock_add(Evas_Object *parent)
    Evas *e;
    Widget_Data *wd;
 
+   EINA_SAFETY_ON_NULL_RETURN_VAL(parent, NULL);
+
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
@@ -518,8 +543,9 @@ elm_clock_add(Evas_Object *parent)
    elm_widget_theme_hook_set(obj, _theme_hook);
    elm_widget_on_focus_hook_set(obj, _on_focus_hook, NULL);
    elm_widget_signal_emit_hook_set(obj, _signal_emit_hook);
-   elm_widget_signal_listen_hook_set(obj, _signal_listen_hook);
-   elm_widget_signal_unlisten_hook_set(obj, _signal_unlisten_hook);
+   elm_widget_signal_callback_add_hook_set(obj, _signal_callback_add_hook);
+   elm_widget_signal_callback_del_hook_set(obj, _signal_callback_del_hook);
+   elm_widget_can_focus_set(obj, EINA_TRUE);
 
    wd->clk = edje_object_add(e);
    elm_widget_resize_object_set(obj, wd->clk);
@@ -529,6 +555,8 @@ elm_clock_add(Evas_Object *parent)
    wd->cur.am_pm = EINA_TRUE;
    wd->cur.edit = EINA_TRUE;
    wd->cur.digedit = ELM_CLOCK_NONE;
+   wd->first_interval = 0.85;
+   wd->timediff = 0;
 
    _time_update(obj);
    _ticker(obj);
@@ -557,6 +585,7 @@ elm_clock_time_set(Evas_Object *obj, int hrs, int min, int sec)
    wd->hrs = hrs;
    wd->min = min;
    wd->sec = sec;
+   _timediff_set(wd);
    _time_update(obj);
 }
 
@@ -607,7 +636,9 @@ elm_clock_edit_set(Evas_Object *obj, Eina_Bool edit)
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
    wd->edit = edit;
-   if (edit && (wd->digedit == ELM_CLOCK_NONE))
+   if (!edit)
+     _timediff_set(wd);
+   if ((edit) && (wd->digedit == ELM_CLOCK_NONE))
      elm_clock_digit_edit_set(obj, ELM_CLOCK_ALL);
    else
      _time_update(obj);
@@ -761,3 +792,45 @@ elm_clock_show_seconds_get(const Evas_Object *obj)
    if (!wd) return EINA_FALSE;
    return wd->seconds;
 }
+
+/**
+ * Set the interval for the clock
+ *
+ * @param obj The clock object
+ * @param interval The interval value in seconds
+ *
+ * The interval value is decreased while the user increments or decrements
+ * the clock value. The next interval value is the previous interval / 1.05,
+ * so it speed up a bit. Default value is 0.85 seconds.
+ *
+ * @ingroup Clock
+ */
+EAPI void
+elm_clock_interval_set(Evas_Object *obj, double interval)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   wd->first_interval = interval;
+}
+
+/**
+ * Get the interval of the clock
+ *
+ * @param obj The clock object
+ * @return The value of the first interval in seconds
+ *
+ * The interval value is decreased while the user increments or decrements
+ * the clock value. The next interval value is the previous interval / 1.05,
+ * so it speed up a bit. Default value is 0.85 seconds.
+ *
+ * @ingroup Clock
+ */
+EAPI double
+elm_clock_interval_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) 0.0;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return 0.0;
+   return wd->first_interval;
+}
