@@ -1,4 +1,5 @@
 #include <Elementary.h>
+#include <Elementary_Cursor.h>
 #include "elm_priv.h"
 
 /**
@@ -10,15 +11,15 @@
  * input.  Entry widgets are capable of expanding past the
  * boundaries of the window, thus resizing the window to its
  * own length.
- * 
+ *
  * You can also insert "items" in the entry with:
- * 
+ *
  * \<item size=16x16 vsize=full href=emoticon/haha\>\</item\>
- * 
+ *
  * for example. sizing can be set bu size=WxH, relsize=WxH or absize=WxH with
  * vsize=ascent or vsize=full. the href=NAME sets the item name. Entry
  * supports a list of emoticon names by default. These are:
- * 
+ *
  * - emoticon/angry
  * - emoticon/angry-shout
  * - emoticon/crazy-laugh
@@ -63,12 +64,17 @@
  * These are built-in currently, but you can add your own item provieer that
  * can create inlined objects in the text and fill the space allocated to the
  * item with a custom object of your own.
- * 
+ *
  * See the entry test for some more examples of use of this.
- * 
+ *
+ * Entries have functions to load a text file, display it,
+ * allowing editing of it and saving of changes back to the file loaded.
+ * Changes are written back to the original file after a short delay.
+ * The file to load and save to is specified by elm_entry_file_set().
+ *
  * Signals that you can add callbacks for are:
  * - "changed" - The text within the entry was changed
- * - "activated" - The entry has received focus and the cursor
+ * - "activated" - The entry has had editing finished and changes are to be committed (generally when enter key is pressed)
  * - "press" - The entry has been clicked
  * - "longpressed" - The entry has been clicked for a couple seconds
  * - "clicked" - The entry has been clicked
@@ -90,6 +96,7 @@ typedef struct _Mod_Api Mod_Api;
 typedef struct _Widget_Data Widget_Data;
 typedef struct _Elm_Entry_Context_Menu_Item Elm_Entry_Context_Menu_Item;
 typedef struct _Elm_Entry_Item_Provider Elm_Entry_Item_Provider;
+typedef struct _Elm_Entry_Text_Filter Elm_Entry_Text_Filter;
 
 struct _Widget_Data
 {
@@ -99,14 +106,18 @@ struct _Widget_Data
    Ecore_Event_Handler *sel_notify_handler;
    Ecore_Event_Handler *sel_clear_handler;
    Ecore_Timer *longpress_timer;
+   Ecore_Timer *delay_write;
    /* Only for clipboard */
    const char *cut_sel;
    const char *text;
+   const char *file;
+   Elm_Text_Format format;
    Evas_Coord lastw;
    Evas_Coord downx, downy;
    Evas_Coord cx, cy, cw, ch;
    Eina_List *items;
    Eina_List *item_providers;
+   Eina_List *text_filters;
    Ecore_Job *hovdeljob;
    Mod_Api *api; // module api if supplied
    Eina_Bool changed : 1;
@@ -121,6 +132,9 @@ struct _Widget_Data
    Eina_Bool deferred_cur : 1;
    Eina_Bool disabled : 1;
    Eina_Bool context_menu : 1;
+   Eina_Bool drag_selection_asked : 1;
+   Eina_Bool can_write : 1;
+   Eina_Bool autosave : 1;
 };
 
 struct _Elm_Entry_Context_Menu_Item
@@ -140,7 +154,15 @@ struct _Elm_Entry_Item_Provider
    void *data;
 };
 
+struct _Elm_Entry_Text_Filter
+{
+   void (*func) (void *data, Evas_Object *entry, char **text);
+   void *data;
+};
+
 static const char *widtype = NULL;
+
+static Eina_Bool _drag_drop_cb(void *data, Evas_Object *obj, Elm_Selection_Data *);
 static void _del_hook(Evas_Object *obj);
 static void _theme_hook(Evas_Object *obj);
 static void _disable_hook(Evas_Object *obj);
@@ -221,12 +243,176 @@ _module(Evas_Object *obj __UNUSED__)
    return m->api;
 }
 
+static char *
+_buf_append(char *buf, const char *str, int *len, int *alloc)
+{
+   int len2 = strlen(str);
+   if ((*len + len2) >= *alloc)
+     {
+        char *buf2 = realloc(buf, *alloc + len2 + 512);
+        if (!buf2) return NULL;
+        buf = buf2;
+        *alloc += (512 + len2);
+     }
+   strcpy(buf + *len, str);
+   *len += len2;
+   return buf;
+}
+
+static char *
+_load_file(const char *file)
+{
+   FILE *f;
+   size_t size;
+   int alloc = 0, len = 0;
+   char *text = NULL, buf[PATH_MAX];
+
+   f = fopen(file, "rb");
+   if (!f) return NULL;
+   while ((size = fread(buf, 1, sizeof(buf), f)))
+     {
+        char *tmp_text;
+        buf[size] = 0;
+        tmp_text = _buf_append(text, buf, &len, &alloc);
+        if (!tmp_text) break;
+        text = tmp_text;
+     }
+   fclose(f);
+   return text;
+}
+
+static char *
+_load_plain(const char *file)
+{
+   char *text;
+
+   text = _load_file(file);
+   if (text)
+     {
+        char *text2;
+
+        text2 = elm_entry_utf8_to_markup(text);
+        free(text);
+        return text2;
+     }
+   return NULL;
+}
+
+static void
+_load(Evas_Object *obj)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   char *text;
+   if (!wd) return;
+   if (!wd->file)
+     {
+        elm_entry_entry_set(obj, "");
+        return;
+     }
+   switch (wd->format)
+     {
+      case ELM_TEXT_FORMAT_PLAIN_UTF8:
+         text = _load_plain(wd->file);
+         break;
+      case ELM_TEXT_FORMAT_MARKUP_UTF8:
+         text = _load_file(wd->file);
+         break;
+      default:
+         text = NULL;
+         break;
+     }
+   if (text)
+     {
+        elm_entry_entry_set(obj, text);
+        free(text);
+     }
+   else
+     elm_entry_entry_set(obj, "");
+}
+
+static void
+_save_markup_utf8(const char *file, const char *text)
+{
+   FILE *f;
+
+   if ((!text) || (!text[0]))
+     {
+        ecore_file_unlink(file);
+        return;
+     }
+   f = fopen(file, "wb");
+   if (!f)
+     {
+        // FIXME: report a write error
+        return;
+     }
+   fputs(text, f); // FIXME: catch error
+   fclose(f);
+}
+
+static void
+_save_plain_utf8(const char *file, const char *text)
+{
+   char *text2;
+
+   text2 = elm_entry_markup_to_utf8(text);
+   if (!text2)
+     return;
+   _save_markup_utf8(file, text2);
+   free(text2);
+}
+
+static void
+_save(Evas_Object *obj)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   if (!wd->file) return;
+   switch (wd->format)
+     {
+      case ELM_TEXT_FORMAT_PLAIN_UTF8:
+	_save_plain_utf8(wd->file, elm_entry_entry_get(obj));
+	break;
+      case ELM_TEXT_FORMAT_MARKUP_UTF8:
+	_save_markup_utf8(wd->file, elm_entry_entry_get(obj));
+	break;
+      default:
+	break;
+     }
+}
+
+static Eina_Bool
+_delay_write(void *data)
+{
+   Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return ECORE_CALLBACK_CANCEL;
+   _save(data);
+   wd->delay_write = NULL;
+   return ECORE_CALLBACK_CANCEL;
+}
+
+static void
+_del_pre_hook(Evas_Object *obj)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   if (wd->delay_write)
+     {
+        ecore_timer_del(wd->delay_write);
+        wd->delay_write = NULL;
+        if (wd->autosave) _save(obj);
+     }
+}
+
 static void
 _del_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Elm_Entry_Context_Menu_Item *it;
    Elm_Entry_Item_Provider *ip;
+   Elm_Entry_Text_Filter *tf;
+
+   if (wd->file) eina_stringshare_del(wd->file);
 
    if (wd->hovdeljob) ecore_job_del(wd->hovdeljob);
    if ((wd->api) && (wd->api->obj_unhook)) wd->api->obj_unhook(obj); // module - unhook
@@ -251,6 +437,10 @@ _del_hook(Evas_Object *obj)
      {
         free(ip);
      }
+   EINA_LIST_FREE(wd->text_filters, tf)
+     {
+        free(tf);
+     }
    free(wd);
 }
 
@@ -264,6 +454,9 @@ _theme_hook(Evas_Object *obj)
    _elm_theme_object_set(obj, wd->ent, "entry", _getbase(obj), elm_widget_style_get(obj));
    elm_entry_entry_set(obj, t);
    eina_stringshare_del(t);
+   if (elm_widget_disabled_get(obj))
+      edje_object_signal_emit(wd->ent, "elm,state,disabled", "elm");
+   edje_object_message_signal_process(wd->ent);
    edje_object_scale_set(wd->ent, elm_widget_scale_get(obj) * _elm_config->scale);
    _sizing_eval(obj);
 }
@@ -295,7 +488,6 @@ _elm_win_recalc_job(void *data)
    wd->deferred_recalc_job = NULL;
    evas_object_geometry_get(wd->ent, NULL, NULL, &resw, &resh);
    resh = 0;
-   minminw = 0;
    edje_object_size_min_restricted_calc(wd->ent, &minw, &minh, 0, 0);
    elm_coords_finger_size_adjust(1, &minw, 1, &minh);
    minminw = minw;
@@ -315,7 +507,7 @@ _sizing_eval(Evas_Object *obj)
    Evas_Coord minw = -1, minh = -1, maxw = -1, maxh = -1;
    Evas_Coord resw, resh;
    if (!wd) return;
-   if (wd->linewrap || wd->char_linewrap)
+   if ((wd->linewrap) || (wd->char_linewrap))
      {
 	evas_object_geometry_get(wd->ent, NULL, NULL, &resw, &resh);
 	if ((resw == wd->lastw) && (!wd->changed)) return;
@@ -344,7 +536,7 @@ _on_focus_hook(void *data __UNUSED__, Evas_Object *obj)
    if (!wd->editable) return;
    if (elm_widget_focus_get(obj))
      {
-	evas_object_focus_set(wd->ent, 1);
+	evas_object_focus_set(wd->ent, EINA_TRUE);
 	edje_object_signal_emit(wd->ent, "elm,action,focus", "elm");
 	if (top) elm_win_keyboard_mode_set(top, ELM_WIN_KEYBOARD_ON);
 	evas_object_smart_callback_call(obj, SIG_FOCUSED, NULL);
@@ -352,7 +544,7 @@ _on_focus_hook(void *data __UNUSED__, Evas_Object *obj)
    else
      {
 	edje_object_signal_emit(wd->ent, "elm,action,unfocus", "elm");
-	evas_object_focus_set(wd->ent, 0);
+	evas_object_focus_set(wd->ent, EINA_FALSE);
 	if (top) elm_win_keyboard_mode_set(top, ELM_WIN_KEYBOARD_OFF);
 	evas_object_smart_callback_call(obj, SIG_UNFOCUSED, NULL);
      }
@@ -366,6 +558,29 @@ _signal_emit_hook(Evas_Object *obj, const char *emission, const char *source)
    edje_object_signal_emit(wd->ent, emission, source);
 }
 
+static void
+_signal_callback_add_hook(Evas_Object *obj, const char *emission, const char *source, void (*func_cb) (void *data, Evas_Object *o, const char *emission, const char *source), void *data)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   edje_object_signal_callback_add(wd->ent, emission, source, func_cb, data);
+}
+
+static void
+_signal_callback_del_hook(Evas_Object *obj, const char *emission, const char *source, void (*func_cb) (void *data, Evas_Object *o, const char *emission, const char *source), void *data)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   edje_object_signal_callback_del_full(wd->ent, emission, source, func_cb,
+                                        data);
+}
+
+static void
+_on_focus_region_hook(const Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, Evas_Coord *w, Evas_Coord *h)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   edje_object_part_text_cursor_geometry_get(wd->ent, "elm.text", x, y, w, h);
+}
+
 static void
 _hoversel_position(Evas_Object *obj)
 {
@@ -403,7 +618,7 @@ _resize(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event
 {
    Widget_Data *wd = elm_widget_data_get(data);
    if (!wd) return;
-   if (wd->linewrap || wd->char_linewrap)
+   if ((wd->linewrap) || (wd->char_linewrap))
      {
         _sizing_eval(data);
      }
@@ -435,7 +650,7 @@ _dismissed(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
    if (wd->selmode)
      {
         if (!wd->password)
-          edje_object_part_text_select_allow_set(wd->ent, "elm.text", 1);
+          edje_object_part_text_select_allow_set(wd->ent, "elm.text", EINA_TRUE);
      }
    elm_widget_scroll_freeze_pop(data);
    if (wd->hovdeljob) ecore_job_del(wd->hovdeljob);
@@ -450,7 +665,7 @@ _select(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
    wd->selmode = EINA_TRUE;
    edje_object_part_text_select_none(wd->ent, "elm.text");
    if (!wd->password)
-     edje_object_part_text_select_allow_set(wd->ent, "elm.text", 1);
+     edje_object_part_text_select_allow_set(wd->ent, "elm.text", EINA_TRUE);
    edje_object_signal_emit(wd->ent, "elm,state,select,on", "elm");
    elm_widget_scroll_hold_push(data);
 }
@@ -464,20 +679,22 @@ _paste(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
    if (wd->sel_notify_handler)
      {
 #ifdef HAVE_ELEMENTARY_X
-	elm_selection_get(ELM_SEL_CLIPBOARD, ELM_SEL_MARKUP, data);
+        wd->selection_asked = EINA_TRUE;
+	elm_selection_get(ELM_SEL_CLIPBOARD, ELM_SEL_FORMAT_MARKUP, data,
+                          NULL, NULL);
 #endif
      }
 }
 
 static void
-_store_selection(enum _elm_sel_type seltype, Evas_Object *obj)
+_store_selection(Elm_Sel_Type seltype, Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    const char *sel;
 
    if (!wd) return;
    sel = edje_object_part_text_selection_get(wd->ent, "elm.text");
-   elm_selection_set(seltype, obj, ELM_SEL_MARKUP, sel);
+   elm_selection_set(seltype, obj, ELM_SEL_FORMAT_MARKUP, sel);
    if (seltype == ELM_SEL_CLIPBOARD)
 	   eina_stringshare_replace(&wd->cut_sel, sel);
 }
@@ -553,7 +770,7 @@ _long_press(void *data)
           (wd->ent, "context_menu_orientation");
         if ((context_menu_orientation) &&
             (!strcmp(context_menu_orientation, "horizontal")))
-          elm_hoversel_horizontal_set(wd->hoversel, 1);
+          elm_hoversel_horizontal_set(wd->hoversel, EINA_TRUE);
         elm_object_style_set(wd->hoversel, "entry");
         elm_widget_sub_object_add(data, wd->hoversel);
         elm_hoversel_label_set(wd->hoversel, "Text");
@@ -745,6 +962,10 @@ _str_append(char *str, const char *txt, int *len, int *alloc)
    return str;
 }
 
+/*FIXME: Sholud be implemented somewhere else, it really depends on the context
+ * because some markups can be implemented otherwise according to style.
+ * probably doing it in textblock and making it translate according to it's
+ * style is correct. */
 static char *
 _mkup_to_text(const char *mkup)
 {
@@ -759,7 +980,7 @@ _mkup_to_text(const char *mkup)
    s = p;
    for (;;)
      {
-	if ((*p == 0) ||
+	if ((!*p) ||
 	    (tag_end) || (esc_end) ||
 	    (tag_start) || (esc_start))
 	  {
@@ -782,6 +1003,8 @@ _mkup_to_text(const char *mkup)
 			 str = _str_append(str, "\t", &str_len, &str_alloc);
 		       else if (!strcmp(ttag, "\\t"))
 			 str = _str_append(str, "\t", &str_len, &str_alloc);
+		       else if (!strcmp(ttag, "ps")) /* Unicode paragraph separator */
+			 str = _str_append(str, "\xE2\x80\xA9", &str_len, &str_alloc);
 		       free(ttag);
 		    }
 		  tag_start = tag_end = NULL;
@@ -801,7 +1024,7 @@ _mkup_to_text(const char *mkup)
 		    }
 		  esc_start = esc_end = NULL;
 	       }
-	     else if (*p == 0)
+	     else if ((!*p) && (s))
 	       {
 		  ts = malloc(p - s + 1);
 		  if (ts)
@@ -816,7 +1039,7 @@ _mkup_to_text(const char *mkup)
 	  }
 	if (*p == '<')
 	  {
-	     if (!esc_start)
+	     if ((s) && (!esc_start))
 	       {
 		  tag_start = p;
 		  tag_end = NULL;
@@ -841,7 +1064,7 @@ _mkup_to_text(const char *mkup)
 	  }
 	else if (*p == '&')
 	  {
-	     if (!tag_start)
+	     if ((s) && (!tag_start))
 	       {
 		  esc_start = p;
 		  esc_end = NULL;
@@ -893,6 +1116,8 @@ _text_to_mkup(const char *text)
           str = _str_append(str, "&gt;", &str_len, &str_alloc);
 	else if (ch == '&')
           str = _str_append(str, "&amp;", &str_len, &str_alloc);
+        else if (ch == 0x2029) /* PS */
+          str = _str_append(str, "<ps>", &str_len, &str_alloc);
 	else
 	  {
 	     char tstr[16];
@@ -915,6 +1140,13 @@ _signal_entry_changed(void *data, Evas_Object *obj __UNUSED__, const char *emiss
    if (wd->text) eina_stringshare_del(wd->text);
    wd->text = NULL;
    evas_object_smart_callback_call(data, SIG_CHANGED, NULL);
+   if (wd->delay_write)
+     {
+	ecore_timer_del(wd->delay_write);
+	wd->delay_write = NULL;
+     }
+   if ((!wd->autosave) || (!wd->file)) return;
+   wd->delay_write = ecore_timer_add(2.0, _delay_write, data);
 }
 
 static void
@@ -938,7 +1170,7 @@ _signal_selection_start(void *data, Evas_Object *obj __UNUSED__, const char *emi
 
 	top = elm_widget_top_get(data);
 	if ((top) && (elm_win_xwindow_get(top)))
-	     elm_selection_set(ELM_SEL_PRIMARY, data, ELM_SEL_MARKUP, txt);
+	     elm_selection_set(ELM_SEL_PRIMARY, data, ELM_SEL_FORMAT_MARKUP, txt);
      }
 #endif
 }
@@ -950,7 +1182,7 @@ _signal_selection_changed(void *data, Evas_Object *obj __UNUSED__, const char *e
    if (!wd) return;
    wd->have_selection = EINA_TRUE;
    evas_object_smart_callback_call(data, SIG_SELECTION_CHANGED, NULL);
-   elm_selection_set(ELM_SEL_PRIMARY, obj, ELM_SEL_MARKUP,
+   elm_selection_set(ELM_SEL_PRIMARY, obj, ELM_SEL_FORMAT_MARKUP,
 		   elm_entry_selection_get(data));
 }
 
@@ -971,7 +1203,7 @@ _signal_selection_cleared(void *data, Evas_Object *obj __UNUSED__, const char *e
 
 	     top = elm_widget_top_get(data);
 	     if ((top) && (elm_win_xwindow_get(top)))
-	         elm_selection_set(ELM_SEL_PRIMARY, data, ELM_SEL_MARKUP,
+	         elm_selection_set(ELM_SEL_PRIMARY, data, ELM_SEL_FORMAT_MARKUP,
 				       wd->cut_sel);
 #endif
 	     eina_stringshare_del(wd->cut_sel);
@@ -1004,9 +1236,9 @@ _signal_entry_paste_request(void *data, Evas_Object *obj __UNUSED__, const char
 	top = elm_widget_top_get(data);
 	if ((top) && (elm_win_xwindow_get(top)))
 	  {
-	     ecore_x_selection_primary_request(elm_win_xwindow_get(top),
-                                               ECORE_X_SELECTION_TARGET_UTF8_STRING);
-	     wd->selection_asked = EINA_TRUE;
+             wd->selection_asked = EINA_TRUE;
+             elm_selection_get(ELM_SEL_CLIPBOARD, ELM_SEL_FORMAT_MARKUP, data,
+                               NULL, NULL);
 	  }
 #endif
      }
@@ -1018,7 +1250,7 @@ _signal_entry_copy_notify(void *data, Evas_Object *obj __UNUSED__, const char *e
    Widget_Data *wd = elm_widget_data_get(data);
    if (!wd) return;
    evas_object_smart_callback_call(data, SIG_SELECTION_COPY, NULL);
-   elm_selection_set(ELM_SEL_CLIPBOARD, obj, ELM_SEL_MARKUP,
+   elm_selection_set(ELM_SEL_CLIPBOARD, obj, ELM_SEL_FORMAT_MARKUP,
 			elm_entry_selection_get(data));
 }
 
@@ -1028,7 +1260,7 @@ _signal_entry_cut_notify(void *data, Evas_Object *obj __UNUSED__, const char *em
    Widget_Data *wd = elm_widget_data_get(data);
    if (!wd) return;
    evas_object_smart_callback_call(data, SIG_SELECTION_CUT, NULL);
-   elm_selection_set(ELM_SEL_CLIPBOARD, obj, ELM_SEL_MARKUP,
+   elm_selection_set(ELM_SEL_CLIPBOARD, obj, ELM_SEL_FORMAT_MARKUP,
 			elm_entry_selection_get(data));
    edje_object_part_text_insert(wd->ent, "elm.text", "");
    wd->changed = EINA_TRUE;
@@ -1155,7 +1387,7 @@ _signal_mouse_down(void *data, Evas_Object *obj __UNUSED__, const char *emission
 }
 
 static void
-_signal_mouse_up(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
+_signal_mouse_clicked(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
    if (!wd) return;
@@ -1177,7 +1409,8 @@ _event_selection_notify(void *data, int type __UNUSED__, void *event)
    Widget_Data *wd = elm_widget_data_get(data);
    Ecore_X_Event_Selection_Notify *ev = event;
    if (!wd) return ECORE_CALLBACK_PASS_ON;
-   if (!wd->selection_asked) return ECORE_CALLBACK_PASS_ON;
+   if ((!wd->selection_asked) && (!wd->drag_selection_asked))
+      return ECORE_CALLBACK_PASS_ON;
 
    if ((ev->selection == ECORE_X_SELECTION_CLIPBOARD) ||
        (ev->selection == ECORE_X_SELECTION_PRIMARY))
@@ -1200,17 +1433,41 @@ _event_selection_notify(void *data, int type __UNUSED__, void *event)
 	  }
 	wd->selection_asked = EINA_FALSE;
      }
+   else if (ev->selection == ECORE_X_SELECTION_XDND)
+     {
+	Ecore_X_Selection_Data_Text *text_data;
+
+	text_data = ev->data;
+	if (text_data->data.content == ECORE_X_SELECTION_CONTENT_TEXT)
+	  {
+	     if (text_data->text)
+	       {
+		  char *txt = _text_to_mkup(text_data->text);
+
+		  if (txt)
+		    {
+                     /* Massive FIXME: this should be at the drag point */
+		       elm_entry_entry_insert(data, txt);
+		       free(txt);
+		    }
+	       }
+	  }
+	wd->drag_selection_asked = EINA_FALSE;
+
+        ecore_x_dnd_send_finished();
+
+     }
    return ECORE_CALLBACK_PASS_ON;
 }
 
 static Eina_Bool
-_event_selection_clear(void *data, int type __UNUSED__, void *event)
+_event_selection_clear(void *data __UNUSED__, int type __UNUSED__, void *event __UNUSED__)
 {
+/*
    Widget_Data *wd = elm_widget_data_get(data);
    Ecore_X_Event_Selection_Clear *ev = event;
-/*
-   if (!wd) return 1;
-   if (!wd->have_selection) return 1;
+   if (!wd) return ECORE_CALLBACK_PASS_ON;
+   if (!wd->have_selection) return ECORE_CALLBACK_PASS_ON;
    if ((ev->selection == ECORE_X_SELECTION_CLIPBOARD) ||
        (ev->selection == ECORE_X_SELECTION_PRIMARY))
      {
@@ -1219,30 +1476,87 @@ _event_selection_clear(void *data, int type __UNUSED__, void *event)
    return 1;*/
    return ECORE_CALLBACK_PASS_ON;
 }
+
+
+static Eina_Bool
+_drag_drop_cb(void *data __UNUSED__, Evas_Object *obj, Elm_Selection_Data *drop)
+{
+   Widget_Data *wd;
+   Eina_Bool rv;
+
+   wd = elm_widget_data_get(obj);
+
+   if (!wd) return EINA_FALSE;
+   printf("Inserting at (%d,%d) %s\n",drop->x,drop->y,(char*)drop->data);
+
+   edje_object_part_text_cursor_copy(wd->ent, "elm.text",
+                                     EDJE_CURSOR_MAIN,/*->*/EDJE_CURSOR_USER);
+   rv = edje_object_part_text_cursor_coord_set(wd->ent,"elm.text",
+                                          EDJE_CURSOR_MAIN,drop->x,drop->y);
+   if (!rv) printf("Warning: Failed to position cursor: paste anyway\n");
+   elm_entry_entry_insert(obj, drop->data);
+   edje_object_part_text_cursor_copy(wd->ent, "elm.text",
+                                     EDJE_CURSOR_USER,/*->*/EDJE_CURSOR_MAIN);
+
+   return EINA_TRUE;
+}
 #endif
 
 static Evas_Object *
-_get_item(void *data, Evas_Object *edje, const char *part, const char *item)
+_get_item(void *data, Evas_Object *edje __UNUSED__, const char *part __UNUSED__, const char *item)
 {
    Widget_Data *wd = elm_widget_data_get(data);
    Evas_Object *o;
    Eina_List *l;
    Elm_Entry_Item_Provider *ip;
-   int ok = 0;
 
    EINA_LIST_FOREACH(wd->item_providers, l, ip)
      {
         o = ip->func(ip->data, data, item);
         if (o) return o;
      }
+   if (!strncmp(item, "file://", 7))
+     {
+        char *fname = item + 7;
+       
+        o = evas_object_image_filled_add(evas_object_evas_get(data));
+        evas_object_image_file_set(o, fname, NULL);
+        if (evas_object_image_load_error_get(o) == EVAS_LOAD_ERROR_NONE)
+          {
+             evas_object_show(o);
+          }
+        else
+          {
+             evas_object_del(o);
+             o = edje_object_add(evas_object_evas_get(data));
+             _elm_theme_object_set(data, o, "entry/emoticon", "wtf", elm_widget_style_get(data));
+          }
+        return o;
+     }
    o = edje_object_add(evas_object_evas_get(data));
-   if (!strncmp(item, "emoticon/", 9))
-     ok = _elm_theme_object_set(data, o, "entry", item, elm_widget_style_get(data));
-   if (!ok)
+   if (!_elm_theme_object_set(data, o, "entry", item, elm_widget_style_get(data)))
      _elm_theme_object_set(data, o, "entry/emoticon", "wtf", elm_widget_style_get(data));
    return o;
 }
 
+static void
+_text_filter(void *data, Evas_Object *edje __UNUSED__, const char *part __UNUSED__, Edje_Text_Filter_Type type, char **text)
+{
+   Widget_Data *wd = elm_widget_data_get(data);
+   Eina_List *l;
+   Elm_Entry_Text_Filter *tf;
+
+   if (type == EDJE_TEXT_FILTER_FORMAT)
+     return;
+
+   EINA_LIST_FOREACH(wd->text_filters, l, tf)
+     {
+        tf->func(tf->data, data, text);
+        if (!*text)
+           break;
+     }
+}
+
 /**
  * This adds an entry to @p parent object.
  *
@@ -1258,6 +1572,8 @@ elm_entry_add(Evas_Object *parent)
    Evas *e;
    Widget_Data *wd;
 
+   EINA_SAFETY_ON_NULL_RETURN_VAL(parent, NULL);
+
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
@@ -1267,19 +1583,27 @@ elm_entry_add(Evas_Object *parent)
    elm_widget_on_focus_hook_set(obj, _on_focus_hook, NULL);
    elm_widget_data_set(obj, wd);
    elm_widget_del_hook_set(obj, _del_hook);
+   elm_widget_del_pre_hook_set(obj, _del_pre_hook);
    elm_widget_theme_hook_set(obj, _theme_hook);
    elm_widget_disable_hook_set(obj, _disable_hook);
    elm_widget_signal_emit_hook_set(obj, _signal_emit_hook);
-   elm_widget_can_focus_set(obj, 1);
+   elm_widget_on_focus_region_hook_set(obj, _on_focus_region_hook);
+   elm_widget_signal_callback_add_hook_set(obj, _signal_callback_add_hook);
+   elm_widget_signal_callback_del_hook_set(obj, _signal_callback_del_hook);
+   elm_object_cursor_set(obj, ELM_CURSOR_XTERM);
+   elm_widget_can_focus_set(obj, EINA_TRUE);
+   elm_widget_highlight_ignore_set(obj, EINA_TRUE);
 
    wd->linewrap     = EINA_TRUE;
    wd->char_linewrap= EINA_FALSE;
    wd->editable     = EINA_TRUE;
    wd->disabled     = EINA_FALSE;
    wd->context_menu = EINA_TRUE;
+   wd->autosave     = EINA_TRUE;
 
    wd->ent = edje_object_add(e);
    edje_object_item_provider_set(wd->ent, _get_item, obj);
+   edje_object_text_insert_filter_callback_add(wd->ent,"elm.text", _text_filter, obj);
    evas_object_event_callback_add(wd->ent, EVAS_CALLBACK_MOVE, _move, obj);
    evas_object_event_callback_add(wd->ent, EVAS_CALLBACK_RESIZE, _resize, obj);
    evas_object_event_callback_add(wd->ent, EVAS_CALLBACK_MOUSE_DOWN,
@@ -1320,8 +1644,8 @@ elm_entry_add(Evas_Object *parent)
                                    _signal_key_enter, obj);
    edje_object_signal_callback_add(wd->ent, "mouse,down,1", "elm.text",
                                    _signal_mouse_down, obj);
-   edje_object_signal_callback_add(wd->ent, "mouse,up,1", "elm.text",
-                                   _signal_mouse_up, obj);
+   edje_object_signal_callback_add(wd->ent, "mouse,clicked,1", "elm.text",
+                                   _signal_mouse_clicked, obj);
    edje_object_signal_callback_add(wd->ent, "mouse,down,1,double", "elm.text",
                                    _signal_mouse_double, obj);
    edje_object_part_text_set(wd->ent, "elm.text", "");
@@ -1339,6 +1663,8 @@ elm_entry_add(Evas_Object *parent)
 	  ecore_event_handler_add(ECORE_X_EVENT_SELECTION_CLEAR,
 				  _event_selection_clear, obj);
      }
+
+   elm_drop_target_add(obj, ELM_SEL_FORMAT_MARKUP,_drag_drop_cb, NULL);
 #endif
 
    entries = eina_list_prepend(entries, obj);
@@ -1517,8 +1843,13 @@ elm_entry_selection_get(const Evas_Object *obj)
 }
 
 /**
- * This inserts text in @p entry at the beginning of the entry
- * object.
+ * This inserts text in @p entry where the current cursor position.
+ * 
+ * This inserts text at the cursor position is as if it was typed 
+ * by the user (note this also allows markup which a user
+ * can't just "type" as it would be converted to escaped text, so this
+ * call can be used to insert things like emoticon items or bold push/pop
+ * tags, other font and color change tags etc.)
  *
  * @param obj The entry object
  * @param entry The text to insert
@@ -1620,6 +1951,13 @@ elm_entry_editable_set(Evas_Object *obj, Eina_Bool editable)
    elm_entry_entry_set(obj, t);
    eina_stringshare_del(t);
    _sizing_eval(obj);
+
+#ifdef HAVE_ELEMENTARY_X
+   if (editable)
+      elm_drop_target_add(obj, ELM_SEL_FORMAT_MARKUP, _drag_drop_cb, NULL);
+   else
+      elm_drop_target_del(obj);
+#endif
 }
 
 /**
@@ -2062,7 +2400,7 @@ elm_entry_item_provider_append(Evas_Object *obj, Evas_Object *(*func) (void *dat
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
-   if (!func) return;
+   EINA_SAFETY_ON_NULL_RETURN(func);
    Elm_Entry_Item_Provider *ip = calloc(1, sizeof(Elm_Entry_Item_Provider));
    if (!ip) return;
    ip->func = func;
@@ -2088,7 +2426,7 @@ elm_entry_item_provider_prepend(Evas_Object *obj, Evas_Object *(*func) (void *da
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
-   if (!func) return;
+   EINA_SAFETY_ON_NULL_RETURN(func);
    Elm_Entry_Item_Provider *ip = calloc(1, sizeof(Elm_Entry_Item_Provider));
    if (!ip) return;
    ip->func = func;
@@ -2116,7 +2454,7 @@ elm_entry_item_provider_remove(Evas_Object *obj, Evas_Object *(*func) (void *dat
    Eina_List *l;
    Elm_Entry_Item_Provider *ip;
    if (!wd) return;
-   if (!func) return;
+   EINA_SAFETY_ON_NULL_RETURN(func);
    EINA_LIST_FOREACH(wd->item_providers, l, ip)
      {
         if ((ip->func == func) && (ip->data == data))
@@ -2128,6 +2466,106 @@ elm_entry_item_provider_remove(Evas_Object *obj, Evas_Object *(*func) (void *dat
      }
 }
 
+/**
+ * Append a filter function for text inserted in the entry
+ *
+ * Append the given callback to the list. This functions will be called
+ * whenever any text is inserted into the entry, with the text to be inserted
+ * as a parameter. The callback function is free to alter the text in any way
+ * it wants, but it must remember to free the given pointer and update it.
+ * If the new text is to be discarded, the function can free it and set it text
+ * parameter to NULL. This will also prevent any following filters from being
+ * called.
+ *
+ * @param obj The entry object
+ * @param func The function to use as text filter
+ * @param data User data to pass to @p func
+ *
+ * @ingroup Entry
+ */
+EAPI void
+elm_entry_text_filter_append(Evas_Object *obj, void (*func) (void *data, Evas_Object *entry, char **text), void *data)
+{
+   Widget_Data *wd;
+   Elm_Entry_Text_Filter *tf;
+   ELM_CHECK_WIDTYPE(obj, widtype);
+
+   wd = elm_widget_data_get(obj);
+
+   EINA_SAFETY_ON_NULL_RETURN(func);
+
+   tf = ELM_NEW(Elm_Entry_Text_Filter);
+   if (!tf) return;
+   tf->func = func;
+   tf->data = data;
+   wd->text_filters = eina_list_append(wd->text_filters, tf);
+}
+
+/**
+ * Prepend a filter function for text insdrted in the entry
+ *
+ * Prepend the given callback to the list. See elm_entry_text_filter_append()
+ * for more information
+ *
+ * @param obj The entry object
+ * @param func The function to use as text filter
+ * @param data User data to pass to @p func
+ *
+ * @ingroup Entry
+ */
+EAPI void
+elm_entry_text_filter_prepend(Evas_Object *obj, void (*func) (void *data, Evas_Object *entry, char **text), void *data)
+{
+   Widget_Data *wd;
+   Elm_Entry_Text_Filter *tf;
+   ELM_CHECK_WIDTYPE(obj, widtype);
+
+   wd = elm_widget_data_get(obj);
+
+   EINA_SAFETY_ON_NULL_RETURN(func);
+
+   tf = ELM_NEW(Elm_Entry_Text_Filter);
+   if (!tf) return;
+   tf->func = func;
+   tf->data = data;
+   wd->text_filters = eina_list_prepend(wd->text_filters, tf);
+}
+
+/**
+ * Remove a filter from the list
+ *
+ * Removes the given callback from the filter list. See elm_entry_text_filter_append()
+ * for more information.
+ *
+ * @param obj The entry object
+ * @param func The filter function to remove
+ * @param data The user data passed when adding the function
+ *
+ * @ingroup Entry
+ */
+EAPI void
+elm_entry_text_filter_remove(Evas_Object *obj, void (*func) (void *data, Evas_Object *entry, char **text), void *data)
+{
+   Widget_Data *wd;
+   Eina_List *l;
+   Elm_Entry_Text_Filter *tf;
+   ELM_CHECK_WIDTYPE(obj, widtype);
+
+   wd = elm_widget_data_get(obj);
+
+   EINA_SAFETY_ON_NULL_RETURN(func);
+
+   EINA_LIST_FOREACH(wd->text_filters, l, tf)
+     {
+        if ((tf->func == func) && (tf->data == data))
+          {
+             wd->text_filters = eina_list_remove_list(wd->text_filters, l);
+             free(tf);
+             return;
+          }
+     }
+}
+
 /**
  * This converts a markup (HTML-like) string into UTF-8.
  *
@@ -2159,3 +2597,267 @@ elm_entry_utf8_to_markup(const char *s)
    if (!ss) ss = strdup("");
    return ss;
 }
+
+/**
+ * Filter inserted text based on user defined character and byte limits
+ *
+ * Add this filter to an entry to limit the characters that it will accept
+ * based the the contents of the provided Elm_Entry_Filter_Limit_Size.
+ * The funtion works on the UTF-8 representation of the string, converting
+ * it from the set markup, thus not accounting for any format in it.
+ *
+ * The user must create an Elm_Entry_Filter_Limit_Size structure and pass
+ * it as data when setting the filter. In it it's possible to set limits
+ * by character count or bytes (any of them is disabled if 0), and both can
+ * be set at the same time. In that case, it first checks for characters,
+ * then bytes.
+ *
+ * The function will cut the inserted text in order to allow only the first
+ * number of characters that are still allowed. The cut is made in
+ * characters, even when limiting by bytes, in order to always contain
+ * valid ones and avoid half unicode characters making it in.
+ *
+ * @ingroup Entry
+ */
+EAPI void
+elm_entry_filter_limit_size(void *data, Evas_Object *entry, char **text)
+{
+   Elm_Entry_Filter_Limit_Size *lim = data;
+   char *current;
+   int len, newlen;
+   const char *(*text_get)(const Evas_Object *);
+   const char *widget_type;
+
+   EINA_SAFETY_ON_NULL_RETURN(data);
+   EINA_SAFETY_ON_NULL_RETURN(entry);
+   EINA_SAFETY_ON_NULL_RETURN(text);
+
+   /* hack. I don't want to copy the entire function to work with
+    * scrolled_entry */
+   widget_type = elm_widget_type_get(entry);
+   if (!strcmp(widget_type, "entry"))
+      text_get = elm_entry_entry_get;
+   else if (!strcmp(widget_type, "scrolled_entry"))
+      text_get = elm_scrolled_entry_entry_get;
+   else /* huh? */
+      return;
+
+   current = elm_entry_markup_to_utf8(text_get(entry));
+
+   if (lim->max_char_count > 0)
+     {
+        int cut;
+        len = evas_string_char_len_get(current);
+        if (len >= lim->max_char_count)
+          {
+             free(*text);
+             free(current);
+             *text = NULL;
+             return;
+          }
+        newlen = evas_string_char_len_get(*text);
+        cut = strlen(*text);
+        while ((len + newlen) > lim->max_char_count)
+          {
+             cut = evas_string_char_prev_get(*text, cut, NULL);
+             newlen--;
+          }
+        (*text)[cut] = 0;
+     }
+
+   if (lim->max_byte_count > 0)
+     {
+        len = strlen(current);
+        if (len >= lim->max_byte_count)
+          {
+             free(*text);
+             free(current);
+             *text = NULL;
+             return;
+          }
+        newlen = strlen(*text);
+        while ((len + newlen) > lim->max_byte_count)
+          {
+             int p = evas_string_char_prev_get(*text, newlen, NULL);
+             newlen -= (newlen - p);
+          }
+        if (newlen)
+           (*text)[newlen] = 0;
+        else
+          {
+             free(*text);
+             *text = NULL;
+          }
+     }
+   free(current);
+}
+
+/**
+ * Filter inserted text based on accepted or rejected sets of characters
+ *
+ * Add this filter to an entry to restrict the set of accepted characters
+ * based on the sets in the provided Elm_Entry_Filter_Accept_Set.
+ * This structure contains both accepted and rejected sets, but they are
+ * mutually exclusive. If accepted is set, it will be used, otherwise it
+ * goes on to the rejected set.
+ */
+EAPI void
+elm_entry_filter_accept_set(void *data, Evas_Object *entry __UNUSED__, char **text)
+{
+   Elm_Entry_Filter_Accept_Set *as = data;
+   const char *set;
+   char *insert;
+   Eina_Bool goes_in;
+   int read_idx, last_read_idx = 0, read_char;
+
+   EINA_SAFETY_ON_NULL_RETURN(data);
+   EINA_SAFETY_ON_NULL_RETURN(text);
+
+   if ((!as->accepted) && (!as->rejected))
+      return;
+
+   if (as->accepted)
+     {
+        set = as->accepted;
+        goes_in = EINA_TRUE;
+     }
+   else
+     {
+        set = as->rejected;
+        goes_in = EINA_FALSE;
+     }
+
+   insert = *text;
+   read_idx = evas_string_char_next_get(*text, 0, &read_char);
+   while (read_char)
+     {
+        int cmp_idx, cmp_char;
+        Eina_Bool in_set = EINA_FALSE;
+
+        cmp_idx = evas_string_char_next_get(set, 0, &cmp_char);
+        while (cmp_char)
+          {
+             if (read_char == cmp_char)
+               {
+                  in_set = EINA_TRUE;
+                  break;
+               }
+             cmp_idx = evas_string_char_next_get(set, cmp_idx, &cmp_char);
+          }
+        if (in_set == goes_in)
+          {
+             int size = read_idx - last_read_idx;
+             const char *src = (*text) + last_read_idx;
+             if (src != insert)
+                memcpy(insert, *text + last_read_idx, size);
+             insert += size;
+          }
+        last_read_idx = read_idx;
+        read_idx = evas_string_char_next_get(*text, read_idx, &read_char);
+     }
+   *insert = 0;
+}
+
+/**
+ * This sets the file (and implicitly loads it) for the text to display and
+ * then edit. All changes are written back to the file after a short delay if
+ * the entry object is set to autosave.
+ *
+ * @param obj The entry object
+ * @param file The path to the file to load and save
+ * @param format The file format
+ *
+ * @ingroup Entry
+ */
+EAPI void
+elm_entry_file_set(Evas_Object *obj, const char *file, Elm_Text_Format format)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   if (wd->delay_write)
+     {
+	ecore_timer_del(wd->delay_write);
+	wd->delay_write = NULL;
+     }
+   if (wd->autosave) _save(obj);
+   eina_stringshare_replace(&wd->file, file);
+   wd->format = format;
+   _load(obj);
+}
+
+/**
+ * Gets the file to load and save and the file format
+ *
+ * @param obj The entry object
+ * @param file The path to the file to load and save
+ * @param format The file format
+ *
+ * @ingroup Entry
+ */
+EAPI void
+elm_entry_file_get(const Evas_Object *obj, const char **file, Elm_Text_Format *format)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   if (file) *file = wd->file;
+   if (format) *format = wd->format;
+}
+
+/**
+ * This function writes any changes made to the file set with
+ * elm_entry_file_set()
+ *
+ * @param obj The entry object
+ *
+ * @ingroup Entry
+ */
+EAPI void
+elm_entry_file_save(Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   if (wd->delay_write)
+     {
+	ecore_timer_del(wd->delay_write);
+	wd->delay_write = NULL;
+     }
+   _save(obj);
+   wd->delay_write = ecore_timer_add(2.0, _delay_write, obj);
+}
+
+/**
+ * This sets the entry object to 'autosave' the loaded text file or not.
+ *
+ * @param obj The entry object
+ * @param autosave Autosave the loaded file or not
+ *
+ * @ingroup Entry
+ */
+EAPI void
+elm_entry_autosave_set(Evas_Object *obj, Eina_Bool autosave)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   wd->autosave = !!autosave;
+}
+
+/**
+ * This gets the entry object's 'autosave' status.
+ *
+ * @param obj The entry object
+ * @return Autosave the loaded file or not
+ *
+ * @ingroup Entry
+ */
+EAPI Eina_Bool
+elm_entry_autosave_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
+   return wd->autosave;
+}
