@@ -14,6 +14,7 @@ struct _Widget_Data
 {
    Evas_Object *frm;
    Evas_Object *content;
+   const char *label;
 };
 
 static const char *widtype = NULL;
@@ -28,6 +29,7 @@ _del_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
+   if (wd->label) eina_stringshare_del(wd->label);
    free(wd);
 }
 
@@ -37,12 +39,28 @@ _theme_hook(Evas_Object *obj)
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
    _elm_theme_object_set(obj, wd->frm, "frame", "base", elm_widget_style_get(obj));
+   edje_object_part_text_set(wd->frm, "elm.text", wd->label);
    if (wd->content)
      edje_object_part_swallow(wd->frm, "elm.swallow.content", wd->content);
    edje_object_scale_set(wd->frm, elm_widget_scale_get(obj) * _elm_config->scale);
    _sizing_eval(obj);
 }
 
+static Eina_Bool
+_elm_frame_focus_next_hook(const Evas_Object *obj, Elm_Focus_Direction dir, Evas_Object **next)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   Evas_Object *cur;
+
+   if ((!wd) || (!wd->content))
+     return EINA_FALSE;
+
+   cur = wd->content;
+
+   /* Try Focus cycle in subitem */
+   return elm_widget_focus_next_get(cur, dir, next);
+}
+
 static void
 _sizing_eval(Evas_Object *obj)
 {
@@ -95,6 +113,8 @@ elm_frame_add(Evas_Object *parent)
    Evas *e;
    Widget_Data *wd;
 
+   EINA_SAFETY_ON_NULL_RETURN_VAL(parent, NULL);
+
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
@@ -104,6 +124,8 @@ elm_frame_add(Evas_Object *parent)
    elm_widget_data_set(obj, wd);
    elm_widget_del_hook_set(obj, _del_hook);
    elm_widget_theme_hook_set(obj, _theme_hook);
+   elm_widget_focus_next_hook_set(obj, _elm_frame_focus_next_hook);
+   elm_widget_can_focus_set(obj, EINA_FALSE);
 
    wd->frm = edje_object_add(e);
    _elm_theme_object_set(obj, wd->frm, "frame", "base", "default");
@@ -129,7 +151,8 @@ elm_frame_label_set(Evas_Object *obj, const char *label)
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
-   edje_object_part_text_set(wd->frm, "elm.text", label);
+   eina_stringshare_replace(&(wd->label), label);
+   edje_object_part_text_set(wd->frm, "elm.text", wd->label);
    _sizing_eval(obj);
 }
 
@@ -147,8 +170,8 @@ elm_frame_label_get(const Evas_Object *obj)
 {
    ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
-   if ((!wd) || (!wd->frm)) return NULL;
-   return edje_object_part_text_get(wd->frm, "elm.text");
+   if (!wd) return NULL;
+   return wd->label;
 }
 
 /**
@@ -182,6 +205,25 @@ elm_frame_content_set(Evas_Object *obj, Evas_Object *content)
    _sizing_eval(obj);
 }
 
+/**
+ * Get the content of the frame widget
+ *
+ * Return the content object which is set for this widget
+ *
+ * @param obj The frame object
+ * @return The content that is being used
+ *
+ * @ingroup Frame
+ */
+EAPI Evas_Object *
+elm_frame_content_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
+   return wd->content;
+}
+
 /**
  * Unset the content of the frame widget
  *
