@@ -4,17 +4,11 @@
 /**
  * @defgroup Index Index
  *
- * XXX
- *
- * Signals that you can add callbacks for are:
- *
- * xxx - XXX.
- * 
- * xxx - XXX.
+ * An index object is a type of list that categorizes items in it
+ * by letter.
  */
 
 typedef struct _Widget_Data Widget_Data;
-typedef struct _Item Item;
 
 struct _Widget_Data
 {
@@ -31,38 +25,36 @@ struct _Widget_Data
    Eina_Bool down : 1;
 };
 
-struct _Item
+struct _Elm_Index_Item
 {
-   Evas_Object *obj;
+   Elm_Widget_Item base;
    const char *letter;
-   const void *data;
    int level;
-   Evas_Object *base;
    Eina_Bool selected : 1;
 };
 
-static void _del_hook(Evas_Object *obj);
+static const char *widtype = NULL;
 static void _theme_hook(Evas_Object *obj);
 static void _sizing_eval(Evas_Object *obj);
 static void _index_box_auto_fill(Evas_Object *obj, Evas_Object *box, int level);
 static void _index_box_clear(Evas_Object *obj, Evas_Object *box, int level);
-static void _item_free(Item *it);
+static void _item_free(Elm_Index_Item *it);
+
+static void
+_del_pre_hook(Evas_Object *obj)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   _index_box_clear(obj, wd->bx[wd->level], wd->level);
+   _index_box_clear(obj, wd->bx[0], 0);
+   while (wd->items) _item_free(wd->items->data);
+   if (wd->delay) ecore_timer_del(wd->delay);
+}
 
 static void
 _del_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-   Item *it;
-   Eina_List *l, *clear = NULL;
-
-   if (!wd) return;
-   _index_box_clear(obj, wd->bx[wd->level], wd->level);
-   _index_box_clear(obj, wd->bx[0], 0);
-   EINA_LIST_FOREACH(wd->items, l, it)
-     clear = eina_list_append(clear, it);
-   EINA_LIST_FREE(clear, it)
-     _item_free(it);
-   if (wd->delay) ecore_timer_del(wd->delay);
    free(wd);
 }
 
@@ -70,22 +62,49 @@ static void
 _layout(Evas_Object *o, Evas_Object_Box_Data *priv, void *data)
 {
    Widget_Data *wd = data;
-
+   if (!wd) return;
    _els_box_layout(o, priv, wd->horizontal, 1);
 }
 
+static void
+_signal_emit_hook(Evas_Object *obj, const char *emission, const char *source)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   edje_object_signal_emit(wd->base, emission, source);
+}
+
+static void
+_signal_callback_add_hook(Evas_Object *obj, const char *emission, const char *source, void (*func_cb) (void *data, Evas_Object *o, const char *emission, const char *source), void *data)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   edje_object_signal_callback_add(wd->base, emission, source, func_cb, data);
+}
+
+static void
+_signal_callback_del_hook(Evas_Object *obj, const char *emission, const char *source, void (*func_cb) (void *data, Evas_Object *o, const char *emission, const char *source), void *data)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   edje_object_signal_callback_del_full(wd->base, emission, source, func_cb,
+                                        data);
+}
+
 static void
 _theme_hook(Evas_Object *obj)
 {
+   Evas_Coord minw = 0, minh = 0;
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return;
    _index_box_clear(obj, wd->bx[0], 0);
    _index_box_clear(obj, wd->bx[1], 1);
    if (wd->horizontal)
-     _elm_theme_set(wd->base, "index", "base/horizontal", elm_widget_style_get(obj));
+     _elm_theme_object_set(obj, wd->base, "index", "base/horizontal", elm_widget_style_get(obj));
    else
-     _elm_theme_set(wd->base, "index", "base/vertical", elm_widget_style_get(obj));
+     _elm_theme_object_set(obj, wd->base, "index", "base/vertical", elm_widget_style_get(obj));
    edje_object_part_swallow(wd->base, "elm.swallow.event.0", wd->event[0]);
+   elm_coords_finger_size_adjust(1, &minw, 1, &minh);
+   evas_object_size_hint_min_set(wd->event[0], minw, minh);
    edje_object_part_swallow(wd->base, "elm.swallow.index.0", wd->bx[0]);
    if (edje_object_part_exists(wd->base, "elm.swallow.index.1"))
      {
@@ -107,16 +126,12 @@ _theme_hook(Evas_Object *obj)
      {
         if (!wd->event[1])
           {
-             Evas_Coord minw, minh;
-
              wd->event[1] = evas_object_rectangle_add(evas_object_evas_get(wd->base));
              evas_object_color_set(wd->event[1], 0, 0, 0, 0);
-             evas_object_size_hint_min_set(wd->event[1], minw, minh);
-             minw = minh = 0;
-             elm_coords_finger_size_adjust(1, &minw, 1, &minh);
              elm_widget_sub_object_add(obj, wd->event[1]);
           }
         edje_object_part_swallow(wd->base, "elm.swallow.event.1", wd->event[1]);
+        evas_object_size_hint_min_set(wd->event[1], minw, minh);
      }
    else if (wd->event[1])
      {
@@ -137,50 +152,47 @@ _sizing_eval(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord minw = -1, minh = -1, maxw = -1, maxh = -1;
-
-   elm_coords_finger_size_adjust(1, &minw, 1, &minh);
-   edje_object_size_min_restricted_calc(wd->base, &minw, &minh, minw, minh);
-   elm_coords_finger_size_adjust(1, &minw, 1, &minh);
+   if (!wd) return;
+   edje_object_size_min_calc(wd->base, &minw, &minh);
    evas_object_size_hint_min_set(obj, minw, minh);
    evas_object_size_hint_max_set(obj, maxw, maxh);
 }
 
-static Item *
+static Elm_Index_Item *
 _item_new(Evas_Object *obj, const char *letter, const void *item)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-   Item *it;
-
-   it = calloc(1, sizeof(Item));
+   Elm_Index_Item *it;
+   if (!wd) return NULL;
+   it = elm_widget_item_new(obj, Elm_Index_Item);
    if (!it) return NULL;
-   it->obj = obj;
    it->letter = eina_stringshare_add(letter);
-   it->data = item;
+   it->base.data = item;
    it->level = wd->level;
    return it;
 }
 
-static Item *
+static Elm_Index_Item *
 _item_find(Evas_Object *obj, const void *item)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Eina_List *l;
-   Item *it;
-
+   Elm_Index_Item *it;
+   if (!wd) return NULL;
    EINA_LIST_FOREACH(wd->items, l, it)
-     if (it->data == item) return it;
+     if (it->base.data == item) return it;
    return NULL;
 }
 
 static void
-_item_free(Item *it)
+_item_free(Elm_Index_Item *it)
 {
-   Widget_Data *wd = elm_widget_data_get(it->obj);
-
+   Widget_Data *wd = elm_widget_data_get(it->base.widget);
+   if (!wd) return;
    wd->items = eina_list_remove(wd->items, it);
-   if (it->base) evas_object_del(it->base);
+   elm_widget_item_pre_notify_del(it);
    eina_stringshare_del(it->letter);
-   free(it);
+   elm_widget_item_del(it);
 }
 
 // FIXME: always have index filled
@@ -189,10 +201,10 @@ _index_box_auto_fill(Evas_Object *obj, Evas_Object *box, int level)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Eina_List *l;
-   Item *it, **itfit = NULL;
+   Elm_Index_Item *it;
    Evas_Coord mw, mh, w, h;
    int i = 0;
-
+   if (!wd) return;
    if (wd->level_active[level]) return;
    evas_object_geometry_get(box, NULL, NULL, &w, &h);
    EINA_LIST_FOREACH(wd->items, l, it)
@@ -202,16 +214,16 @@ _index_box_auto_fill(Evas_Object *obj, Evas_Object *box, int level)
 
         if (it->level != level) continue;
         o = edje_object_add(evas_object_evas_get(obj));
-        it->base = o;
+        it->base.view = o;
         if (i & 0x1)
-          _elm_theme_set(o, "index", "item_odd/vertical", "default");
+          _elm_theme_object_set(obj, o, "index", "item_odd/vertical", elm_widget_style_get(obj));
         else
-          _elm_theme_set(o, "index", "item/vertical", "default");
+          _elm_theme_object_set(obj, o, "index", "item/vertical", elm_widget_style_get(obj));
         edje_object_part_text_set(o, "elm.text", it->letter);
         edje_object_size_min_restricted_calc(o, &mw, &mh, 0, 0);
         evas_object_size_hint_min_set(o, mw, mh);
-        evas_object_size_hint_weight_set(o, 1.0, 1.0);
-        evas_object_size_hint_align_set(o, -1.0, -1.0);
+        evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
+        evas_object_size_hint_align_set(o, EVAS_HINT_FILL, EVAS_HINT_FILL);
         elm_widget_sub_object_add(obj, o);
         evas_object_box_append(box, o);
         stacking = edje_object_data_get(o, "stacking");
@@ -238,48 +250,47 @@ _index_box_auto_fill(Evas_Object *obj, Evas_Object *box, int level)
 }
 
 static void
-_index_box_clear(Evas_Object *obj, Evas_Object *box, int level)
+_index_box_clear(Evas_Object *obj, Evas_Object *box __UNUSED__, int level)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Eina_List *l;
-   Item *it;
-
+   Elm_Index_Item *it;
+   if (!wd) return;
    if (!wd->level_active[level]) return;
    EINA_LIST_FOREACH(wd->items, l, it)
      {
-        if (!it->base) continue;
+        if (!it->base.view) continue;
         if (it->level != level) continue;
-        evas_object_del(it->base);
-        it->base = 0;
+        evas_object_del(it->base.view);
+        it->base.view = NULL;
      }
    wd->level_active[level] = 0;
 }
 
-static int
+static Eina_Bool
 _delay_change(void *data)
 {
    Widget_Data *wd = elm_widget_data_get(data);
    void *d;
-
+   if (!wd) return ECORE_CALLBACK_CANCEL;
    wd->delay = NULL;
    d = (void *)elm_index_item_selected_get(data, wd->level);
    if (d) evas_object_smart_callback_call(data, "delay,changed", d);
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
 static void
 _sel_eval(Evas_Object *obj, Evas_Coord evx, Evas_Coord evy)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-   Item *it, *it_closest, *it_last;
+   Elm_Index_Item *it, *it_closest, *it_last;
    Eina_List *l;
    Evas_Coord x, y, w, h, bx, by, bw, bh, xx, yy;
    double cdv = 0.5;
    Evas_Coord dist;
-   Eina_Bool change = 0;
    char *label = NULL, *last = NULL;
    int i;
-
+   if (!wd) return;
    for (i = 0; i <= wd->level; i++)
      {
         it_last = NULL;
@@ -288,8 +299,8 @@ _sel_eval(Evas_Object *obj, Evas_Coord evx, Evas_Coord evy)
         evas_object_geometry_get(wd->bx[i], &bx, &by, &bw, &bh);
         EINA_LIST_FOREACH(wd->items, l, it)
           {
-             if (!((it->level == i) && (it->base))) continue;
-             if ((it->base) && (it->level != wd->level))
+             if (!((it->level == i) && (it->base.view))) continue;
+             if ((it->base.view) && (it->level != wd->level))
                {
                   if (it->selected)
                     {
@@ -303,7 +314,7 @@ _sel_eval(Evas_Object *obj, Evas_Coord evx, Evas_Coord evy)
                   it_last = it;
                   it->selected = 0;
                }
-             evas_object_geometry_get(it->base, &x, &y, &w, &h);
+             evas_object_geometry_get(it->base.view, &x, &y, &w, &h);
              xx = x + (w / 2);
              yy = y + (h / 2);
              x = evx - xx;
@@ -319,25 +330,24 @@ _sel_eval(Evas_Object *obj, Evas_Coord evx, Evas_Coord evy)
                   dist = x;
                }
           }
-        if ((i == 0) && (wd->level == 0))
+        if ((!i) && (!wd->level))
           edje_object_part_drag_value_set(wd->base, "elm.dragable.index.1", 
                                           cdv, cdv);
         if (it_closest) it_closest->selected = 1;
         if (it_closest != it_last)
           {
-             change = 1;
              if (it_last)
                {
                   const char *stacking, *selectraise;
 
                   it = it_last;
-                  edje_object_signal_emit(it->base, "elm,state,inactive", "elm");
-                  stacking = edje_object_data_get(it->base, "stacking");
-                  selectraise = edje_object_data_get(it->base, "selectraise");
+                  edje_object_signal_emit(it->base.view, "elm,state,inactive", "elm");
+                  stacking = edje_object_data_get(it->base.view, "stacking");
+                  selectraise = edje_object_data_get(it->base.view, "selectraise");
                   if ((selectraise) && (!strcmp(selectraise, "on")))
                     {
                        if ((stacking) && (!strcmp(stacking, "below")))
-                         evas_object_lower(it->base);
+                         evas_object_lower(it->base.view);
                     }
                }
              if (it_closest)
@@ -345,11 +355,11 @@ _sel_eval(Evas_Object *obj, Evas_Coord evx, Evas_Coord evy)
                   const char *selectraise;
 
                   it = it_closest;
-                  edje_object_signal_emit(it->base, "elm,state,active", "elm");
-                  selectraise = edje_object_data_get(it->base, "selectraise");
+                  edje_object_signal_emit(it->base.view, "elm,state,active", "elm");
+                  selectraise = edje_object_data_get(it->base.view, "selectraise");
                   if ((selectraise) && (!strcmp(selectraise, "on")))
-                    evas_object_raise(it->base);
-                  evas_object_smart_callback_call((void *)obj, "changed", (void *)it->data);
+                    evas_object_raise(it->base.view);
+                  evas_object_smart_callback_call((void *)obj, "changed", (void *)it->base.data);
                   if (wd->delay) ecore_timer_del(wd->delay);
                   wd->delay = ecore_timer_add(0.2, _delay_change, obj);
                }
@@ -364,6 +374,7 @@ _sel_eval(Evas_Object *obj, Evas_Coord evx, Evas_Coord evy)
                   if (!label) label = strdup(last);
                   else
                     {
+                       /* FIXME: realloc return NULL if the request fails */
                        label = realloc(label, strlen(label) + strlen(last) + 1);
                        strcat(label, last);
                     }
@@ -381,20 +392,21 @@ _sel_eval(Evas_Object *obj, Evas_Coord evx, Evas_Coord evy)
 }
 
 static void 
-_wheel(void *data, Evas *e, Evas_Object *o, void *event_info)
+_wheel(void *data, Evas *e __UNUSED__, Evas_Object *o __UNUSED__, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
-   Evas_Event_Mouse_Wheel *ev = event_info;
-   Evas_Object *obj = o;
+//   Evas_Event_Mouse_Wheel *ev = event_info;
+//   Evas_Object *obj = o;
+   if (!wd) return;
 }
 
 static void 
-_mouse_down(void *data, Evas *e, Evas_Object *o, void *event_info)
+_mouse_down(void *data, Evas *e __UNUSED__, Evas_Object *o __UNUSED__, void *event_info)
 {
    Widget_Data *wd = elm_widget_data_get(data);
    Evas_Event_Mouse_Down *ev = event_info;
    Evas_Coord x, y;
-
+   if (!wd) return;
    if (ev->button != 1) return;
    wd->down = 1;
    evas_object_geometry_get(wd->base, &x, &y, NULL, NULL);
@@ -407,12 +419,12 @@ _mouse_down(void *data, Evas *e, Evas_Object *o, void *event_info)
 }
 
 static void 
-_mouse_up(void *data, Evas *e, Evas_Object *o, void *event_info)
+_mouse_up(void *data, Evas *e __UNUSED__, Evas_Object *o __UNUSED__, void *event_info)
 {
    Widget_Data *wd = elm_widget_data_get(data);
    Evas_Event_Mouse_Up *ev = event_info;
    void *d;
-
+   if (!wd) return;
    if (ev->button != 1) return;
    wd->down = 0;
    d = (void *)elm_index_item_selected_get(data, wd->level);
@@ -422,15 +434,13 @@ _mouse_up(void *data, Evas *e, Evas_Object *o, void *event_info)
 }
 
 static void 
-_mouse_move(void *data, Evas *e, Evas_Object *o, void *event_info)
+_mouse_move(void *data, Evas *e __UNUSED__, Evas_Object *o __UNUSED__, void *event_info)
 {
    Widget_Data *wd = elm_widget_data_get(data);
    Evas_Event_Mouse_Move *ev = event_info;
-   Evas_Coord minw = 0, minh = 0, x, y, dx, dy, adx, ady;
-   Eina_List *l;
-   Item *it;
+   Evas_Coord minw = 0, minh = 0, x, y, dx, adx;
    char buf[1024];
-
+   if (!wd) return;
    if (!wd->down) return;
    elm_coords_finger_size_adjust(1, &minw, 1, &minh);
    evas_object_geometry_get(wd->base, &x, &y, NULL, NULL);
@@ -439,18 +449,12 @@ _mouse_move(void *data, Evas *e, Evas_Object *o, void *event_info)
    dx = x - wd->dx;
    adx = dx;
    if (adx < 0) adx = -dx;
-   dy = y - wd->dy;
-   ady = dy;
-   if (ady < 0) ady = -dy;
    edje_object_part_drag_value_set(wd->base, "elm.dragable.pointer", x, y);
-   if (wd->horizontal)
-     {
-     }
-   else
+   if (!wd->horizontal)
      {
         if (adx > minw)
           {
-             if (wd->level == 0)
+             if (!wd->level)
                { 
                   wd->level = 1;
                   snprintf(buf, sizeof(buf), "elm,state,level,%i", wd->level);
@@ -489,19 +493,27 @@ elm_index_add(Evas_Object *parent)
    Widget_Data *wd;
    Evas_Coord minw, minh;
 
+   EINA_SAFETY_ON_NULL_RETURN_VAL(parent, NULL);
+
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "index");
    elm_widget_type_set(obj, "index");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_data_set(obj, wd);
    elm_widget_del_hook_set(obj, _del_hook);
+   elm_widget_del_pre_hook_set(obj, _del_pre_hook);
    elm_widget_theme_hook_set(obj, _theme_hook);
+   elm_widget_signal_emit_hook_set(obj, _signal_emit_hook);
+   elm_widget_signal_callback_add_hook_set(obj, _signal_callback_add_hook);
+   elm_widget_signal_callback_del_hook_set(obj, _signal_callback_del_hook);
+   elm_widget_can_focus_set(obj, EINA_FALSE);
 
    wd->horizontal = EINA_FALSE;
 
    wd->base = edje_object_add(e);
-   _elm_theme_set(wd->base, "index", "base/vertical", "default");
+   _elm_theme_object_set(obj, wd->base, "index", "base/vertical", "default");
    elm_widget_resize_object_set(obj, wd->base);
 
    o = evas_object_rectangle_add(e);
@@ -557,8 +569,8 @@ elm_index_add(Evas_Object *parent)
 EAPI void
 elm_index_active_set(Evas_Object *obj, Eina_Bool active)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return;
    if (wd->active == active) return;
    wd->active = active;
@@ -574,7 +586,7 @@ elm_index_active_set(Evas_Object *obj, Eina_Bool active)
 }
 
 /**
- * @brief To be documented.
+ * Sets the level of the item.
  *
  * @param obj The index object.
  * @param level To be documented.
@@ -584,65 +596,66 @@ elm_index_active_set(Evas_Object *obj, Eina_Bool active)
 EAPI void
 elm_index_item_level_set(Evas_Object *obj, int level)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return;
    if (wd->level == level) return;
    wd->level = level;
 }
 
 /**
- * @brief To be documented.
+ * Gets the level of the item.
  *
  * @param obj The index object
  *
  * @ingroup Index
  */
 EAPI int
-elm_index_item_level_get(Evas_Object *obj)
+elm_index_item_level_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) 0;
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return 0;
    return wd->level;
 }
 
 /**
- * @brief To be documented.
+ * Returns the selected item.
  *
  * @param obj The index object.
  * @param level to be documented.
  *
  * @ingroup Index
  */
-EAPI const void *
-elm_index_item_selected_get(Evas_Object *obj, int level)
+EAPI void *
+elm_index_item_selected_get(const Evas_Object *obj, int level)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
    Eina_List *l;
-   Item *it;
-
+   Elm_Index_Item *it;
    if (!wd) return NULL;
    EINA_LIST_FOREACH(wd->items, l, it)
-     if ((it->selected) && (it->level == level)) return it->data;
+     if ((it->selected) && (it->level == level))
+       return elm_widget_item_data_get(it);
    return NULL;
 }
 
 /**
- * @brief To be documented.
+ * Appends a new item.
  *
  * @param obj The index object.
- * @param letter to be documented.
- * @param item to be documented.
+ * @param letter Letter under which the item should be indexed
+ * @param item The item to put in the index
  *
  * @ingroup Index
  */
 EAPI void
 elm_index_item_append(Evas_Object *obj, const char *letter, const void *item)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-   Item *it;
-
+   Elm_Index_Item *it;
    if (!wd) return;
    it = _item_new(obj, letter, item);
    if (!it) return;
@@ -651,19 +664,20 @@ elm_index_item_append(Evas_Object *obj, const char *letter, const void *item)
 }
 
 /**
- * @brief To be documented.
+ * Prepends a new item.
  *
  * @param obj The index object.
- * @param letter to be documented.
- * @param item to be documented.
+ * @param letter Letter under which the item should be indexed
+ * @param item The item to put in the index
  *
  * @ingroup Index
  */
 EAPI void
 elm_index_item_prepend(Evas_Object *obj, const char *letter, const void *item)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-   Item *it;
+   Elm_Index_Item *it;
 
    if (!wd) return;
    it = _item_new(obj, letter, item);
@@ -673,21 +687,21 @@ elm_index_item_prepend(Evas_Object *obj, const char *letter, const void *item)
 }
 
 /**
- * @brief To be documented.
+ * Append an item after @p relative in letter @p letter.
  *
  * @param obj The index object
- * @param letter to be documented.
- * @param item to be documented.
- * @param relative to be documented.
+ * @param letter Letter under which the item should be indexed
+ * @param item The item to put in the index
+ * @param relative The item to put @p item after
  *
  * @ingroup Index
  */
 EAPI void
 elm_index_item_append_relative(Evas_Object *obj, const char *letter, const void *item, const void *relative)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-   Item *it, *it_rel;
-
+   Elm_Index_Item *it, *it_rel;
    if (!wd) return;
    if (!relative)
      {
@@ -707,21 +721,21 @@ elm_index_item_append_relative(Evas_Object *obj, const char *letter, const void
 }
 
 /**
- * @brief To be documented.
+ * Prepend an item before @p relative in letter @p letter.
  *
- * @param obj The index object.
- * @param letter to be documented.
- * @param item to be documented.
- * @param relative to be documented.
+ * @param obj The index object
+ * @param letter Letter under which the item should be indexed
+ * @param item The item to put in the index
+ * @param relative The item to put @p item before
  *
  * @ingroup Index
  */
 EAPI void
 elm_index_item_prepend_relative(Evas_Object *obj, const char *letter, const void *item, const void *relative)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-   Item *it, *it_rel;
-
+   Elm_Index_Item *it, *it_rel;
    if (!wd) return;
    if (!relative)
      {
@@ -741,19 +755,78 @@ elm_index_item_prepend_relative(Evas_Object *obj, const char *letter, const void
 }
 
 /**
- * @brief To be documented.
+ * Insert a new @p item into the sorted index @p obj in @p letter.
  *
  * @param obj The index object
- * @param item To be documented.
+ * @param letter Letter under which the item should be indexed
+ * @param item The item to put in the index
+ * @param cmp_func The function called for the sort of index items.
+ * @param cmp_data_func The function called for the sort of the data. It will
+ * be used when cmp_func return 0. It means the index item already exists.
+ * So, to decide which data item should be pointed by the index item, a function
+ * to compare them is needed. If this function is not provided, index items
+ * will be duplicated. If cmp_data_func returns a non-negative value, the
+ * previous index item data will be replaced by the inserted @p item. So
+ * if the previous data need to be free, it should be done in this function,
+ * because the reference will be lost.
+ *
+ * @ingroup Index
+ */
+EAPI void
+elm_index_item_sorted_insert(Evas_Object *obj, const char *letter, const void *item, Eina_Compare_Cb cmp_func, Eina_Compare_Cb cmp_data_func)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   Eina_List *lnear;
+   Elm_Index_Item *it;
+   int cmp;
+
+   if (!wd) return;
+   if (!(wd->items))
+     {
+        elm_index_item_append(obj, letter, item);
+        return;
+     }
+
+   it = _item_new(obj, letter, item);
+   if (!it) return;
+
+   lnear = eina_list_search_sorted_near_list(wd->items, cmp_func, it, &cmp);
+   if (cmp < 0)
+     wd->items =  eina_list_append_relative_list(wd->items, it, lnear);
+   else if (cmp > 0)
+     wd->items = eina_list_prepend_relative_list(wd->items, it, lnear);
+   else
+     {
+	/* If cmp_data_func is not provided, append a duplicated item */
+	if (!cmp_data_func)
+	  wd->items =  eina_list_append_relative_list(wd->items, it, lnear);
+	else
+	  {
+	     Elm_Index_Item *p_it = eina_list_data_get(lnear);
+	     if (cmp_data_func(p_it->base.data, it->base.data) >= 0)
+	       p_it->base.data = it->base.data;
+	     _item_free(it);
+	  }
+     }
+
+   _index_box_clear(obj, wd->bx[wd->level], wd->level);
+}
+
+/**
+ * Remove an item from the index.
+ *
+ * @param obj The index object
+ * @param item The item to remove from the index
  *
  * @ingroup Index
  */
 EAPI void
 elm_index_item_del(Evas_Object *obj, const void *item)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-   Item *it;
-
+   Elm_Index_Item *it;
    if (!wd) return;
    it = _item_find(obj, item);
    if (!it) return;
@@ -762,7 +835,25 @@ elm_index_item_del(Evas_Object *obj, const void *item)
 }
 
 /**
- * @brief To be documented.
+ * Find an index item using item data.
+ *
+ * @param obj The index object
+ * @param item The item pointed by index item
+ * @return The index item pointing to @p item
+ *
+ * @ingroup Index
+ */
+EAPI Elm_Index_Item *
+elm_index_item_find(Evas_Object *obj, const void *item)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
+   return _item_find(obj, item);
+}
+
+/**
+ * Clears an index of its items.
  *
  * @param obj The index object.
  *
@@ -771,10 +862,10 @@ elm_index_item_del(Evas_Object *obj, const void *item)
 EAPI void
 elm_index_item_clear(Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-   Item *it;
+   Elm_Index_Item *it;
    Eina_List *l, *clear = NULL;
-
    if (!wd) return;
    _index_box_clear(obj, wd->bx[wd->level], wd->level);
    EINA_LIST_FOREACH(wd->items, l, it)
@@ -782,26 +873,86 @@ elm_index_item_clear(Evas_Object *obj)
         if (it->level != wd->level) continue;
         clear = eina_list_append(clear, it);
      }
-   EINA_LIST_FREE(clear, it)
-     _item_free(it);
+   EINA_LIST_FREE(clear, it) _item_free(it);
 }
 
 /**
- * @brief To be documented.
+ * Go to item at @p level
  *
  * @param obj The index object
- * @param level To be documented.
+ * @param level The index level
  *
  * @ingroup Index
  */
 EAPI void
-elm_index_item_go(Evas_Object *obj, int level)
+elm_index_item_go(Evas_Object *obj, int level __UNUSED__)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-   Item *it;
-
    if (!wd) return;
    _index_box_auto_fill(obj, wd->bx[0], 0);
-   if (wd->level == 1)
-     _index_box_auto_fill(obj, wd->bx[1], 1);
+   if (wd->level == 1) _index_box_auto_fill(obj, wd->bx[1], 1);
 }
+
+/**
+ * Returns the data associated with the item.
+ *
+ * @param it The list item
+ * @return The data associated with @p it
+ *
+ * @ingroup Index
+ */
+EAPI void *
+elm_index_item_data_get(const Elm_Index_Item *it)
+{
+   ELM_WIDGET_ITEM_WIDTYPE_CHECK_OR_RETURN(it, NULL);
+   return elm_widget_item_data_get(it);
+}
+
+/**
+ * Set the data item from the index item
+ *
+ * This set a new data value.
+ *
+ * @param it The item
+ * @param data The new data pointer to set
+ *
+ * @ingroup Index
+ */
+EAPI void
+elm_index_item_data_set(Elm_Index_Item *it, const void *data)
+{
+   ELM_WIDGET_ITEM_WIDTYPE_CHECK_OR_RETURN(it);
+   elm_widget_item_data_set(it, data);
+}
+
+/**
+ * Set the function called when a index item is freed.
+ *
+ * @param it The item to set the callback on
+ * @param func The function called
+ *
+ * @ingroup Index
+ */
+EAPI void
+elm_index_item_del_cb_set(Elm_Index_Item *it, Evas_Smart_Cb func)
+{
+   ELM_WIDGET_ITEM_WIDTYPE_CHECK_OR_RETURN(it);
+   elm_widget_item_del_cb_set(it, func);
+}
+
+/**
+ * Gets the letter of the item.
+ *
+ * @param it The list item
+ * @return The letter of @p it
+ *
+ * @ingroup Index
+ */
+EAPI const char *
+elm_index_item_letter_get(const Elm_Index_Item *it)
+{
+   ELM_WIDGET_ITEM_WIDTYPE_CHECK_OR_RETURN(it, NULL);
+   return it->letter;
+}
+
