@@ -1,6 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
 #include <Elementary.h>
 #include "elm_priv.h"
 
@@ -8,8 +5,11 @@
 /**
  * @defgroup Notify notify
  *
- * Display a window in a particular region of the application (top, bottom ...).
- * A timeout can be set to automatically close the window.
+ * Display a window in a particular region of the application (top,
+ * bottom, etc.  A timeout can be set to automatically close the
+ * window. This is so that, after an evas_object_show() on a notify
+ * object, if a timeout was set on it, it will <b>automatically</b>
+ * get hidden after that time.
  *
  */
 
@@ -23,7 +23,7 @@ struct _Widget_Data
    Eina_Bool repeat_events;
    Evas_Object *block_events;
 
-   int timeout;
+   double timeout;
    Ecore_Timer *timer;
 };
 
@@ -40,6 +40,14 @@ static void _show(void *data, Evas *e, Evas_Object *obj, void *event_info);
 static void _hide(void *data, Evas *e, Evas_Object *obj, void *event_info);
 static void _resize(void *data, Evas *e, Evas_Object *obj, void *event_info);
 
+static const char SIG_BLOCK_CLICKED[] = "block,clicked";
+static const char SIG_TIMEOUT[] = "timeout";
+static const Evas_Smart_Cb_Description _signals[] = {
+  {SIG_BLOCK_CLICKED, ""},
+  {SIG_TIMEOUT, ""},
+  {NULL, NULL}
+};
+
 static void
 _del_pre_hook(Evas_Object *obj)
 {
@@ -64,14 +72,61 @@ _del_hook(Evas_Object *obj)
    free(wd);
 }
 
+static void
+_notify_theme_apply(Evas_Object *obj)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   const char *style = elm_widget_style_get(obj);
+
+   switch (wd->orient)
+     {
+      case ELM_NOTIFY_ORIENT_TOP:
+        _elm_theme_object_set(obj, wd->notify, "notify", "top", style);
+        break;
+      case ELM_NOTIFY_ORIENT_CENTER:
+        _elm_theme_object_set(obj, wd->notify, "notify", "center", style);
+        break;
+      case ELM_NOTIFY_ORIENT_BOTTOM:
+        _elm_theme_object_set(obj, wd->notify, "notify", "bottom", style);
+        break;
+      case ELM_NOTIFY_ORIENT_LEFT:
+        _elm_theme_object_set(obj, wd->notify, "notify", "left", style);
+        break;
+      case ELM_NOTIFY_ORIENT_RIGHT:
+        _elm_theme_object_set(obj, wd->notify, "notify", "right", style);
+        break;
+      case ELM_NOTIFY_ORIENT_TOP_LEFT:
+        _elm_theme_object_set(obj, wd->notify, "notify", "top_left", style);
+        break;
+      case ELM_NOTIFY_ORIENT_TOP_RIGHT:
+        _elm_theme_object_set(obj, wd->notify, "notify", "top_right", style);
+        break;
+      case ELM_NOTIFY_ORIENT_BOTTOM_LEFT:
+        _elm_theme_object_set(obj, wd->notify, "notify", "bottom_left", style);
+        break;
+      case ELM_NOTIFY_ORIENT_BOTTOM_RIGHT:
+        _elm_theme_object_set(obj, wd->notify, "notify", "bottom_right", style);
+        break;
+      case ELM_NOTIFY_ORIENT_LAST:
+        break;
+     }
+}
+
+static void
+_block_events_theme_apply(Evas_Object *obj)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   const char *style = elm_widget_style_get(obj);
+   _elm_theme_object_set(obj, wd->block_events, "notify", "block_events", style);
+}
+
 static void
 _theme_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
-   _elm_theme_object_set(obj, wd->notify, "notify", "base", "default");
-   if (wd->block_events)
-     _elm_theme_object_set(obj, wd->block_events, "notify", "block_events", "default");
+   _notify_theme_apply(obj);
+   if (wd->block_events) _block_events_theme_apply(obj);
    edje_object_scale_set(wd->notify, elm_widget_scale_get(obj) *
                          _elm_config->scale);
    _sizing_eval(obj);
@@ -117,7 +172,7 @@ _signal_block_clicked(void *data, Evas_Object *obj __UNUSED__, const char *emiss
 {
    Widget_Data *wd = elm_widget_data_get(data);
    if (!wd) return;
-   evas_object_smart_callback_call(data, "block,clicked", NULL);
+   evas_object_smart_callback_call(data, SIG_BLOCK_CLICKED, NULL);
 }
 
 static void
@@ -138,9 +193,11 @@ _calc(Evas_Object *obj)
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord minw = -1, minh = -1;
    Evas_Coord x, y, w, h;
+
    if (!wd) return;
    evas_object_geometry_get(obj, &x, &y, &w, &h);
-   edje_object_size_min_calc(wd->notify, &minw, &minh);
+   edje_object_size_min_get(wd->notify, &minw, &minh);
+   edje_object_size_min_restricted_calc(wd->notify, &minw, &minh, minw, minh);
 
    if (wd->content)
      {
@@ -176,6 +233,8 @@ _calc(Evas_Object *obj)
 	   case ELM_NOTIFY_ORIENT_BOTTOM_RIGHT:
              evas_object_move(wd->notify, x + w - minw, y + h - minh);
              break;
+           case ELM_NOTIFY_ORIENT_LAST:
+             break;
 	  }
         evas_object_resize(wd->notify, minw, minh);
      }
@@ -190,9 +249,22 @@ _timer_cb(void *data)
    if (!wd) return ECORE_CALLBACK_CANCEL;
    wd->timer = NULL;
    evas_object_hide(obj);
+   evas_object_smart_callback_call(obj, SIG_TIMEOUT, NULL);
    return ECORE_CALLBACK_CANCEL;
 }
 
+static void
+_timer_init(Evas_Object *obj, Widget_Data *wd)
+{
+   if (wd->timer)
+     {
+	ecore_timer_del(wd->timer);
+	wd->timer = NULL;
+     }
+   if ((evas_object_visible_get(obj)) && (wd->timeout > 0.0))
+     wd->timer = ecore_timer_add(wd->timeout, _timer_cb, obj);
+}
+
 static void
 _show(void *data __UNUSED__, Evas *e __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
@@ -201,13 +273,7 @@ _show(void *data __UNUSED__, Evas *e __UNUSED__, Evas_Object *obj, void *event_i
    evas_object_show(wd->notify);
    if (!wd->repeat_events)
      evas_object_show(wd->block_events);
-   if (wd->timer)
-     {
-	ecore_timer_del(wd->timer);
-	wd->timer = NULL;
-     }
-   if (wd->timeout > 0)
-     wd->timer = ecore_timer_add(wd->timeout, _timer_cb, obj);
+   _timer_init(obj, wd);
 }
 
 static void
@@ -258,6 +324,8 @@ elm_notify_add(Evas_Object *parent)
    Evas *e;
    Widget_Data *wd;
 
+   EINA_SAFETY_ON_NULL_RETURN_VAL(parent, NULL);
+
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
@@ -268,10 +336,12 @@ elm_notify_add(Evas_Object *parent)
    elm_widget_del_pre_hook_set(obj, _del_pre_hook);
    elm_widget_del_hook_set(obj, _del_hook);
    elm_widget_theme_hook_set(obj, _theme_hook);
+   elm_widget_can_focus_set(obj, EINA_FALSE);
 
    wd->repeat_events = EINA_TRUE;
 
    wd->notify = edje_object_add(e);
+   wd->orient = -1;
    elm_notify_orient_set(obj, ELM_NOTIFY_ORIENT_TOP);
 
    elm_notify_parent_set(obj, parent);
@@ -283,6 +353,8 @@ elm_notify_add(Evas_Object *parent)
    evas_object_event_callback_add(obj, EVAS_CALLBACK_HIDE, _hide, obj);
 
    _sizing_eval(obj);
+
+   evas_object_smart_callbacks_descriptions_set(obj, _signals);
    return obj;
 }
 
@@ -346,9 +418,30 @@ elm_notify_content_unset(Evas_Object *obj)
    return content;
 }
 
+/**
+ * Return the content of the notify widget
+ *
+ * @param obj The notify object
+ * @return The content that is being used
+ *
+ * @ingroup Notify
+ */
+EAPI Evas_Object *
+elm_notify_content_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+
+   if (!wd) return NULL;
+   return wd->content;
+}
+
 /**
  * Set the notify parent
  *
+ * Once the parent object is set, a previously set one will be desconected
+ * and replaced.
+ *
  * @param obj The notify object
  * @param content The new parent
  *
@@ -396,6 +489,23 @@ elm_notify_parent_set(Evas_Object *obj, Evas_Object *parent)
    _calc(obj);
 }
 
+/**
+ * Get the notify parent
+ *
+ * @param obj The notify object
+ * @return The parent
+ *
+ * @ingroup Notify
+ */
+EAPI Evas_Object *
+elm_notify_parent_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
+   return wd->parent;
+}
+
 /**
  * Set the orientation
  *
@@ -408,71 +518,65 @@ elm_notify_orient_set(Evas_Object *obj, Elm_Notify_Orient orient)
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
+   if (wd->orient == orient) return;
    wd->orient = orient;
-   switch (orient)
-     {
-     case ELM_NOTIFY_ORIENT_TOP:
-        _elm_theme_object_set(obj, wd->notify, "notify", "base", "default");
-        break;
-     case ELM_NOTIFY_ORIENT_CENTER:
-        _elm_theme_object_set(obj, wd->notify, "notify", "base", "center");
-        break;
-     case ELM_NOTIFY_ORIENT_BOTTOM:
-        _elm_theme_object_set(obj, wd->notify, "notify", "base", "bottom");
-        break;
-     case ELM_NOTIFY_ORIENT_LEFT:
-        _elm_theme_object_set(obj, wd->notify, "notify", "base", "left");
-        break;
-     case ELM_NOTIFY_ORIENT_RIGHT:
-        _elm_theme_object_set(obj, wd->notify, "notify", "base", "right");
-        break;
-     case ELM_NOTIFY_ORIENT_TOP_LEFT:
-        _elm_theme_object_set(obj, wd->notify, "notify", "base", "top_left");
-        break;
-     case ELM_NOTIFY_ORIENT_TOP_RIGHT:
-        _elm_theme_object_set(obj, wd->notify, "notify", "base", "top_right");
-        break;
-     case ELM_NOTIFY_ORIENT_BOTTOM_LEFT:
-        _elm_theme_object_set(obj, wd->notify, "notify", "base", "bottom_left");
-        break;
-     case ELM_NOTIFY_ORIENT_BOTTOM_RIGHT:
-        _elm_theme_object_set(obj, wd->notify, "notify", "base", "bottom_right");
-        break;
-     }
+   _notify_theme_apply(obj);
    _resize(obj, NULL, obj, NULL);
 }
 
 /**
- * Set the time before the notify window is hidden. <br>
- * Set a value < 0 to disable the timer
+ * Return the orientation
+ * @param obj the notify objects
+ */
+EAPI Elm_Notify_Orient
+elm_notify_orient_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) -1;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return -1;
+   return wd->orient;
+}
+
+/**
+ * Set the time interval after which the notify window is going to be
+ * hidden.
  *
  * @param obj The notify object
- * @param time the new timeout
+ * @param time The new timeout
+ *
+ * As said previously, an evas_object_show() on a notify object which
+ * had a timeout set by this function will trigger a timer to
+ * automatically hide it again. So, any order one calls
+ * elm_notify_timeout_set() and evas_object_show() on the same object
+ * (at hidden state) will behave the same.
+ *
+ * @note Set a value <= 0.0 to disable a running timer.
+ *
+ * @note If the value > 0.0 and the notify is previously visible, the
+ * timer will be started with this value, canceling any running timer.
+ *
  */
 EAPI void
-elm_notify_timeout_set(Evas_Object *obj, int timeout)
+elm_notify_timeout_set(Evas_Object *obj, double timeout)
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
    wd->timeout = timeout;
-   elm_notify_timer_init(obj);
+   _timer_init(obj, wd);
 }
 
 /**
- * Re-init the timer
- * @param obj The notify object
+ * Return the timeout value (in seconds)
+ * @param obj the notify object
  */
-EAPI void
-elm_notify_timer_init(Evas_Object *obj)
+EAPI double
+elm_notify_timeout_get(const Evas_Object *obj)
 {
-   ELM_CHECK_WIDTYPE(obj, widtype);
+   ELM_CHECK_WIDTYPE(obj, widtype) 0.0;
    Widget_Data *wd = elm_widget_data_get(obj);
-   if (!wd) return;
-   if (wd->timer) ecore_timer_del(wd->timer);
-   wd->timer = NULL;
-   if (wd->timeout > 0)
-     wd->timer = ecore_timer_add(wd->timeout, _timer_cb, obj);
+   if (!wd) return 0.0;
+   return wd->timeout;
 }
 
 /**
@@ -496,11 +600,25 @@ elm_notify_repeat_events_set(Evas_Object *obj, Eina_Bool repeat)
    if (!repeat)
      {
 	wd->block_events = edje_object_add(evas_object_evas_get(obj));
-	_elm_theme_object_set(obj, wd->block_events, "notify", "block_events", "default");
+	_block_events_theme_apply(obj);
         elm_widget_resize_object_set(obj, wd->block_events);
-	edje_object_signal_callback_add(wd->block_events, "elm,action,clicked", "elm", _signal_block_clicked, obj);
+	edje_object_signal_callback_add(wd->block_events, "elm,action,clicked",
+                                        "elm", _signal_block_clicked, obj);
      }
    else
      evas_object_del(wd->block_events);
 }
 
+/**
+ * Return true if events are repeat below the notify object
+ * @param obj the notify object
+ */
+EAPI Eina_Bool
+elm_notify_repeat_events_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
+   return wd->repeat_events;
+}
+
