@@ -136,9 +136,11 @@ struct _Pan
    Widget_Data *wd;
 };
 
+static const char *widtype = NULL;
 static void _del_hook(Evas_Object *obj);
 static void _theme_hook(Evas_Object *obj);
-static void _show_region_hook(void *data, Evas_Object *obj);
+static void _on_focus_hook(void *data, Evas_Object *obj);
+//static void _show_region_hook(void *data, Evas_Object *obj);
 static void _sizing_eval(Evas_Object *obj);
 static void _calc_job(void *data);
 static void grid_place(Evas_Object *obj, Grid *g, Evas_Coord px, Evas_Coord py, Evas_Coord ox, Evas_Coord oy, Evas_Coord ow, Evas_Coord oh);
@@ -163,20 +165,17 @@ img_place(Evas_Object *obj, Evas_Coord px, Evas_Coord py, Evas_Coord ox, Evas_Co
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord ax, ay, gw, gh;
-   int x, y;
-
+   if (!wd) return;
    ax = 0;
    ay = 0;
    gw = wd->size.w;
    gh = wd->size.h;
    if (ow > gw) ax = (ow - gw) / 2;
    if (oh > gh) ay = (oh - gh) / 2;
-   evas_object_move(wd->img, 
-                    ox + 0 - px + ax,
-                    oy + 0 - py + ay);
+   evas_object_move(wd->img, ox + 0 - px + ax, oy + 0 - py + ay);
    evas_object_resize(wd->img, gw, gh);
 
-   if(wd->show.show)
+   if (wd->show.show)
      {
 	wd->show.show = EINA_FALSE;
 	elm_smart_scroller_child_region_show(wd->scr, wd->show.x, wd->show.y, wd->show.w, wd->show.h);
@@ -189,7 +188,7 @@ grid_place(Evas_Object *obj, Grid *g, Evas_Coord px, Evas_Coord py, Evas_Coord o
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord ax, ay, gw, gh, tx, ty;
    int x, y;
-
+   if (!wd) return;
    ax = 0;
    ay = 0;
    gw = wd->size.w;
@@ -232,7 +231,7 @@ grid_clear(Evas_Object *obj, Grid *g)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    int x, y;
-   
+   if (!wd) return;
    if (!g->grid) return;
    for (y = 0; y < g->gh; y++)
      {
@@ -245,7 +244,7 @@ grid_clear(Evas_Object *obj, Grid *g)
              if (g->grid[tn].want)
                {
                   wd->preload_num--;
-                  if (wd->preload_num == 0)
+                  if (!wd->preload_num)
                     {
                        edje_object_signal_emit(elm_smart_scroller_edje_object_get(wd->scr),
                                                "elm,state,busy,stop", "elm");
@@ -261,7 +260,7 @@ grid_clear(Evas_Object *obj, Grid *g)
 }
 
 static void
-_tile_preloaded(void *data, Evas *e, Evas_Object *o, void *event_info)
+_tile_preloaded(void *data, Evas *e __UNUSED__, Evas_Object *o __UNUSED__, void *event_info __UNUSED__)
 {
    Grid_Item *git = data;
 
@@ -271,7 +270,7 @@ _tile_preloaded(void *data, Evas *e, Evas_Object *o, void *event_info)
         evas_object_show(git->img);
         git->have = 1;
         git->wd->preload_num--;
-        if (git->wd->preload_num == 0)
+        if (!git->wd->preload_num)
           {
              edje_object_signal_emit(elm_smart_scroller_edje_object_get(git->wd->scr),
                                      "elm,state,busy,stop", "elm");
@@ -295,6 +294,7 @@ grid_create(Evas_Object *obj)
    int x, y;
    Grid *g;
    
+   if (!wd) return NULL;
    g = calloc(1, sizeof(Grid));
    
    g->zoom = grid_zoom_calc(wd->zoom);
@@ -318,7 +318,7 @@ grid_create(Evas_Object *obj)
      {
         for (x = 0; x < g->gw; x++)
           {
-             int tn, xx, yy, ww, hh;
+             int tn;
              
              tn = (y * g->gw) + x;
              g->grid[tn].src.x = x * g->tsize;
@@ -342,7 +342,7 @@ grid_create(Evas_Object *obj)
                evas_object_image_add(evas_object_evas_get(obj));
              evas_object_image_scale_hint_set
                (g->grid[tn].img, EVAS_IMAGE_SCALE_HINT_DYNAMIC);
-             evas_object_pass_events_set(g->grid[tn].img, 1);
+             evas_object_pass_events_set(g->grid[tn].img, EINA_TRUE);
              evas_object_smart_member_add(g->grid[tn].img, 
                                           wd->pan_smart);
              elm_widget_sub_object_add(obj, g->grid[tn].img);
@@ -361,21 +361,18 @@ grid_load(Evas_Object *obj, Grid *g)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    int x, y;
-   Evas_Coord ow, oh, gw, gh, ax, ay, tx, ty;
-
-   elm_smart_scroller_child_viewport_size_get(wd->scr, &ow, &oh);
-   ax = 0;
-   ay = 0;
+   Evas_Coord ox, oy, ow, oh, cvx, cvy, cvw, cvh, gw, gh, tx, ty;
+   if (!wd) return;
+   evas_object_geometry_get(wd->pan_smart, &ox, &oy, &ow, &oh);
+   evas_output_viewport_get(evas_object_evas_get(wd->obj), &cvx, &cvy, &cvw, &cvh);
    gw = wd->size.w;
    gh = wd->size.h;
-   if (ow > gw) ax = (ow - gw) / 2;
-   if (oh > gh) ay = (oh - gh) / 2;
    for (y = 0; y < g->gh; y++)
      {
         for (x = 0; x < g->gw; x++)
           {
              int tn, xx, yy, ww, hh;
-             Eina_Bool visible = 0;
+             Eina_Bool visible = EINA_FALSE;
              
              tn = (y * g->gw) + x;
              xx = g->grid[tn].out.x;
@@ -394,10 +391,10 @@ grid_load(Evas_Object *obj, Grid *g)
                   yy = (gh * yy) / g->h;
                   hh = ((gh * (ty + hh)) / g->h) - yy;
                }
-//             xx += ax;
-//             yy += ay;
-             if (ELM_RECTS_INTERSECT(wd->pan_x, wd->pan_y, ow, oh,
-                                     xx, yy, ww, hh))
+             if (ELM_RECTS_INTERSECT(xx - wd->pan_x + ox, 
+                                     yy  - wd->pan_y + oy,
+                                     ww, hh,
+                                     cvx, cvy, cvw, cvh))
                visible = 1;
              if ((visible) && (!g->grid[tn].have) && (!g->grid[tn].want))
                {
@@ -423,7 +420,7 @@ grid_load(Evas_Object *obj, Grid *g)
              else if ((g->grid[tn].want) && (!visible))
                {
                   wd->preload_num--;
-                  if (wd->preload_num == 0)
+                  if (!wd->preload_num)
                     {
                        edje_object_signal_emit(elm_smart_scroller_edje_object_get(wd->scr),
                                                "elm,state,busy,stop", "elm");
@@ -450,7 +447,7 @@ grid_clearall(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Grid *g;
-   
+   if (!wd) return;
    EINA_LIST_FREE(wd->grids, g)
      {
         grid_clear(obj, g);
@@ -465,7 +462,7 @@ _smooth_update(Evas_Object *obj)
    int x, y;
    Eina_List *l;
    Grid *g;
-
+   if (!wd) return;
    EINA_LIST_FOREACH(wd->grids, l, g)
      {
         for (y = 0; y < g->gh; y++)
@@ -475,11 +472,11 @@ _smooth_update(Evas_Object *obj)
                   int tn;
                   
                   tn = (y * g->gw) + x;
-                  evas_object_image_smooth_scale_set(g->grid[tn].img, (wd->nosmooth == 0));
+                  evas_object_image_smooth_scale_set(g->grid[tn].img, (!wd->nosmooth));
                }
           }
      }
-   evas_object_image_smooth_scale_set(wd->img, (wd->nosmooth == 0));
+   evas_object_image_smooth_scale_set(wd->img, (!wd->nosmooth));
 }
 
 static void
@@ -499,20 +496,22 @@ _grid_raise(Grid *g)
      }
 }
 
-static int
+static Eina_Bool
 _scr_timeout(void *data)
 {
    Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return ECORE_CALLBACK_CANCEL;
    wd->nosmooth--;
-   if (wd->nosmooth == 0) _smooth_update(data);
+   if (!wd->nosmooth) _smooth_update(data);
    wd->scr_timer = NULL;
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
 static void
-_scr(void *data, Evas_Object *obj, void *event_info)
+_scr(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return;
    if (!wd->scr_timer)
      {
         wd->nosmooth++;
@@ -523,12 +522,12 @@ _scr(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-_main_preloaded(void *data, Evas *e, Evas_Object *o, void *event_info)
+_main_preloaded(void *data, Evas *e __UNUSED__, Evas_Object *o __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *obj = data;
    Widget_Data *wd = elm_widget_data_get(obj);
    Grid *g;
-   
+   if (!wd) return;
    evas_object_show(wd->img);
    wd->main_load_pending = 0;
    g = grid_create(obj);
@@ -541,7 +540,7 @@ _main_preloaded(void *data, Evas *e, Evas_Object *o, void *event_info)
    wd->calc_job = ecore_job_add(_calc_job, wd);
    evas_object_smart_callback_call(data, "loaded", NULL);
    wd->preload_num--;
-   if (wd->preload_num == 0)
+   if (!wd->preload_num)
      {
         edje_object_signal_emit(elm_smart_scroller_edje_object_get(wd->scr),
                                 "elm,state,busy,stop", "elm");
@@ -549,12 +548,12 @@ _main_preloaded(void *data, Evas *e, Evas_Object *o, void *event_info)
      }
 }
 
-static int
+static Eina_Bool
 zoom_do(Evas_Object *obj, double t)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord xx, yy, ow, oh;
-   
+   if (!wd) return ECORE_CALLBACK_CANCEL;
    wd->size.w = (wd->size.ow * (1.0 - t)) + (wd->size.nw * t);
    wd->size.h = (wd->size.oh * (1.0 - t)) + (wd->size.nh * t);
    elm_smart_scroller_child_viewport_size_get(wd->scr, &ow, &oh);
@@ -587,20 +586,20 @@ zoom_do(Evas_Object *obj, double t)
                   free(g);
                }
           }
-        return 0;
+        return ECORE_CALLBACK_CANCEL;
      }
-   return 1;
+   return ECORE_CALLBACK_RENEW;
 }
 
 
-static int
+static Eina_Bool
 _zoom_anim(void *data)
 {
    Evas_Object *obj = data;
    Widget_Data *wd = elm_widget_data_get(obj);
    double t;
-   int go;
-   
+   Eina_Bool go;
+   if (!wd) return ECORE_CALLBACK_CANCEL;
    t = ecore_loop_time_get();
    if (t >= wd->t_end)
      t = 1.0;
@@ -614,7 +613,7 @@ _zoom_anim(void *data)
    if (!go)
      {
         wd->nosmooth--;
-        if (wd->nosmooth == 0) _smooth_update(data);
+        if (!wd->nosmooth) _smooth_update(data);
         wd->zoom_animator = NULL;
         evas_object_smart_callback_call(obj, "zoom,stop", NULL);
      }
@@ -622,27 +621,30 @@ _zoom_anim(void *data)
 }
 
 static void
-_mouse_move(void *data, Evas *evas, Evas_Object *obj, void *event_info)
+_mouse_move(void *data, Evas *evas __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
-   Evas_Event_Mouse_Move *ev = event_info;
+//   Evas_Event_Mouse_Move *ev = event_info;
+   if (!wd) return;
 }
 
-static int
+static Eina_Bool
 _long_press(void *data)
 {
    Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return ECORE_CALLBACK_CANCEL;
    wd->long_timer = NULL;
    wd->longpressed = EINA_TRUE;
    evas_object_smart_callback_call(data, "longpressed", NULL);
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
 static void
-_mouse_down(void *data, Evas *evas, Evas_Object *obj, void *event_info)
+_mouse_down(void *data, Evas *evas __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Widget_Data *wd = elm_widget_data_get(data);
    Evas_Event_Mouse_Down *ev = event_info;
+   if (!wd) return;
    if (ev->button != 1) return;
    if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD) wd->on_hold = EINA_TRUE;
    else wd->on_hold = EINA_FALSE;
@@ -656,10 +658,11 @@ _mouse_down(void *data, Evas *evas, Evas_Object *obj, void *event_info)
 }
    
 static void
-_mouse_up(void *data, Evas *evas, Evas_Object *obj, void *event_info)
+_mouse_up(void *data, Evas *evas __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Widget_Data *wd = elm_widget_data_get(data);
    Evas_Event_Mouse_Up *ev = event_info;
+   if (!wd) return;
    if (ev->button != 1) return;
    if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD) wd->on_hold = EINA_TRUE;
    else wd->on_hold = EINA_FALSE;
@@ -680,7 +683,7 @@ _del_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Grid *g;
-   
+   if (!wd) return;
    EINA_LIST_FREE(wd->grids, g)
      {
         if (g->grid) free(g->grid);
@@ -696,30 +699,51 @@ _del_hook(Evas_Object *obj)
    free(wd);
 }
 
+static void
+_on_focus_hook(void *data __UNUSED__, Evas_Object *obj)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   if (elm_widget_focus_get(obj))
+     {
+       edje_object_signal_emit(wd->obj, "elm,action,focus", "elm");
+       evas_object_focus_set(wd->obj, EINA_TRUE);
+     }
+   else
+     {
+       edje_object_signal_emit(wd->obj, "elm,action,unfocus", "elm");
+       evas_object_focus_set(wd->obj, EINA_FALSE);
+     }
+}
+
 static void
 _theme_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-   elm_smart_scroller_theme_set(wd->scr, "photocam", "base", elm_widget_style_get(obj));
-   edje_object_scale_set(wd->scr, elm_widget_scale_get(obj) * _elm_config->scale);
+   if (!wd) return;
+   elm_smart_scroller_object_theme_set(obj, wd->scr, "photocam", "base", elm_widget_style_get(obj));
+//   edje_object_scale_set(wd->scr, elm_widget_scale_get(obj) * _elm_config->scale);
    _sizing_eval(obj);
 }
 
+/*
 static void
 _show_region_hook(void *data, Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(data);
    Evas_Coord x, y, w, h;
+   if (!wd) return;
    elm_widget_show_region_get(obj, &x, &y, &w, &h);
    elm_smart_scroller_child_region_show(wd->scr, x, y, w, h);
 }
+*/
 
 static void
 _sizing_eval(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord minw = -1, minh = -1, maxw = -1, maxh = -1;
-
+   if (!wd) return;
 //   evas_object_size_hint_min_get(wd->scr, &minw, &minh);
    evas_object_size_hint_max_get(wd->scr, &maxw, &maxh);
 //   minw = -1;
@@ -734,10 +758,9 @@ _calc_job(void *data)
 {
    Widget_Data *wd = data;
    Evas_Coord minw, minh;
-
+   if (!wd) return;
    minw = wd->size.w;
    minh = wd->size.h;
-   
    if (wd->resized)
      {
         wd->resized = 0;
@@ -763,6 +786,7 @@ static void
 _pan_set(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
 {
    Pan *sd = evas_object_smart_data_get(obj);
+   if (!sd) return;
    if ((x == sd->wd->pan_x) && (y == sd->wd->pan_y)) return;
    sd->wd->pan_x = x;
    sd->wd->pan_y = y;
@@ -773,6 +797,7 @@ static void
 _pan_get(Evas_Object *obj, Evas_Coord *x, Evas_Coord *y)
 {
    Pan *sd = evas_object_smart_data_get(obj);
+   if (!sd) return;
    if (x) *x = sd->wd->pan_x;
    if (y) *y = sd->wd->pan_y;
 }
@@ -782,6 +807,7 @@ _pan_max_get(Evas_Object *obj, Evas_Coord *x, Evas_Coord *y)
 {
    Pan *sd = evas_object_smart_data_get(obj);
    Evas_Coord ow, oh;
+   if (!sd) return;
    evas_object_geometry_get(obj, NULL, NULL, &ow, &oh);
    ow = sd->wd->minw - ow;
    if (ow < 0) ow = 0;
@@ -791,10 +817,18 @@ _pan_max_get(Evas_Object *obj, Evas_Coord *x, Evas_Coord *y)
    if (y) *y = oh;
 }
 
+static void
+_pan_min_get(Evas_Object *obj __UNUSED__, Evas_Coord *x, Evas_Coord *y)
+{
+   if (x) *x = 0;
+   if (y) *y = 0;
+}
+
 static void
 _pan_child_size_get(Evas_Object *obj, Evas_Coord *w, Evas_Coord *h)
 {
    Pan *sd = evas_object_smart_data_get(obj);
+   if (!sd) return;
    if (w) *w = sd->wd->minw;
    if (h) *h = sd->wd->minh;
 }
@@ -804,9 +838,9 @@ _pan_add(Evas_Object *obj)
 {
    Pan *sd;
    Evas_Object_Smart_Clipped_Data *cd;
-
    _pan_sc.add(obj);
    cd = evas_object_smart_data_get(obj);
+   if (!cd) return;
    sd = calloc(1, sizeof(Pan));
    if (!sd) return;
    sd->__clipped_data = *cd;
@@ -827,6 +861,7 @@ _pan_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h)
 {
    Pan *sd = evas_object_smart_data_get(obj);
    Evas_Coord ow, oh;
+   if (!sd) return;
    evas_object_geometry_get(obj, NULL, NULL, &ow, &oh);
    if ((ow == w) && (oh == h)) return;
    sd->wd->resized = 1;
@@ -841,7 +876,7 @@ _pan_calculate(Evas_Object *obj)
    Evas_Coord ox, oy, ow, oh;
    Eina_List *l;
    Grid *g;
-   
+   if (!sd) return;
    evas_object_geometry_get(obj, &ox, &oy, &ow, &oh);
    img_place(sd->wd->obj, sd->wd->pan_x, sd->wd->pan_y, ox, oy, ow, oh);
    EINA_LIST_FOREACH(sd->wd->grids, l, g)
@@ -852,7 +887,16 @@ _pan_calculate(Evas_Object *obj)
 }
 
 static void
-_hold_on(void *data, Evas_Object *obj, void *event_info)
+_pan_move(Evas_Object *obj, Evas_Coord x __UNUSED__, Evas_Coord y __UNUSED__)
+{
+   Pan *sd = evas_object_smart_data_get(obj);
+   if (!sd) return;
+   if (sd->wd->calc_job) ecore_job_del(sd->wd->calc_job);
+   sd->wd->calc_job = ecore_job_add(_calc_job, sd->wd);
+}
+
+static void
+_hold_on(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
@@ -860,7 +904,7 @@ _hold_on(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-_hold_off(void *data, Evas_Object *obj, void *event_info)
+_hold_off(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
@@ -868,7 +912,7 @@ _hold_off(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-_freeze_on(void *data, Evas_Object *obj, void *event_info)
+_freeze_on(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
@@ -876,7 +920,7 @@ _freeze_on(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-_freeze_off(void *data, Evas_Object *obj, void *event_info)
+_freeze_off(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
@@ -884,31 +928,31 @@ _freeze_off(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-_scr_anim_start(void *data, Evas_Object *obj, void *event_info)
+_scr_anim_start(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    evas_object_smart_callback_call(data, "scroll,anim,start", NULL);
 }
 
 static void
-_scr_anim_stop(void *data, Evas_Object *obj, void *event_info)
+_scr_anim_stop(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    evas_object_smart_callback_call(data, "scroll,anim,stop", NULL);
 }
 
 static void
-_scr_drag_start(void *data, Evas_Object *obj, void *event_info)
+_scr_drag_start(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    evas_object_smart_callback_call(data, "scroll,drag,start", NULL);
 }
 
 static void
-_scr_drag_stop(void *data, Evas_Object *obj, void *event_info)
+_scr_drag_stop(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    evas_object_smart_callback_call(data, "scroll,drag,stop", NULL);
 }
 
 static void
-_scr_scroll(void *data, Evas_Object *obj, void *event_info)
+_scr_scroll(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    evas_object_smart_callback_call(data, "scroll", NULL);
 }
@@ -929,19 +973,25 @@ elm_photocam_add(Evas_Object *parent)
    Widget_Data *wd;
    Evas_Coord minw, minh;
    static Evas_Smart *smart = NULL;
-   int i;
+   Eina_Bool bounce = _elm_config->thumbscroll_bounce_enable;
+
+   EINA_SAFETY_ON_NULL_RETURN_VAL(parent, NULL);
 
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "photocam");
    elm_widget_type_set(obj, "photocam");
    elm_widget_sub_object_add(parent, obj);
+   elm_widget_on_focus_hook_set(obj, _on_focus_hook, NULL);
    elm_widget_data_set(obj, wd);
    elm_widget_del_hook_set(obj, _del_hook);
    elm_widget_theme_hook_set(obj, _theme_hook);
+   elm_widget_can_focus_set(obj, EINA_TRUE);
 
    wd->scr = elm_smart_scroller_add(e);
-   elm_smart_scroller_theme_set(wd->scr, "photocam", "base", "default");
+   elm_smart_scroller_widget_set(wd->scr, obj);
+   elm_smart_scroller_object_theme_set(obj, wd->scr, "photocam", "base", "default");
    evas_object_smart_callback_add(wd->scr, "scroll", _scr, obj);
    evas_object_smart_callback_add(wd->scr, "drag", _scr, obj);
    elm_widget_resize_object_set(obj, wd->scr);
@@ -952,7 +1002,7 @@ elm_photocam_add(Evas_Object *parent)
    evas_object_smart_callback_add(wd->scr, "drag,stop", _scr_drag_stop, obj);
    evas_object_smart_callback_add(wd->scr, "scroll", _scr_scroll, obj);
    
-   elm_smart_scroller_bounce_allow_set(wd->scr, 1, 1);
+   elm_smart_scroller_bounce_allow_set(wd->scr, bounce, bounce);
 
    wd->obj = obj;
 
@@ -972,6 +1022,7 @@ elm_photocam_add(Evas_Object *parent)
 	sc.add = _pan_add;
 	sc.del = _pan_del;
 	sc.resize = _pan_resize;
+	sc.move = _pan_move;
 	sc.calculate = _pan_calculate;
 	smart = evas_smart_class_new(&sc);
      }
@@ -983,8 +1034,8 @@ elm_photocam_add(Evas_Object *parent)
      }
 
    elm_smart_scroller_extern_pan_set(wd->scr, wd->pan_smart,
-				     _pan_set, _pan_get,
-				     _pan_max_get, _pan_child_size_get);
+				     _pan_set, _pan_get, _pan_max_get,
+                                     _pan_min_get, _pan_child_size_get);
 
    wd->zoom = 1;
    wd->mode = ELM_PHOTOCAM_ZOOM_MODE_MANUAL;
@@ -1030,15 +1081,14 @@ elm_photocam_add(Evas_Object *parent)
  *
  * @ingroup Photocam
  */
-EAPI int
+EAPI Evas_Load_Error
 elm_photocam_file_set(Evas_Object *obj, const char *file)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) EVAS_LOAD_ERROR_NONE;
    Widget_Data *wd = elm_widget_data_get(obj);
    int w, h;
-
-   if (wd->file) eina_stringshare_del(wd->file);
-   
-   wd->file = eina_stringshare_add(file);
+   if (!wd) return EVAS_LOAD_ERROR_GENERIC;
+   if (!eina_stringshare_replace(&wd->file, file)) return EVAS_LOAD_ERROR_NONE;
    evas_object_hide(wd->img);
    evas_object_image_smooth_scale_set(wd->img, (wd->nosmooth == 0));
    evas_object_image_file_set(wd->img, NULL, NULL);
@@ -1088,10 +1138,13 @@ elm_photocam_file_set(Evas_Object *obj, const char *file)
  *
  * @ingroup Photocam
  */
-EAPI const char * elm_photocam_file_get(Evas_Object *obj)
+EAPI const char *
+elm_photocam_file_get(const Evas_Object *obj)
 {
-    Widget_Data *wd = elm_widget_data_get(obj);
-    return wd->file;
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
+   return wd->file;
 }
 
 /**
@@ -1111,6 +1164,7 @@ EAPI const char * elm_photocam_file_get(Evas_Object *obj)
 EAPI void
 elm_photocam_zoom_set(Evas_Object *obj, double zoom)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    Eina_List *l;
    Grid *g, *g_zoom = NULL;
@@ -1118,7 +1172,7 @@ elm_photocam_zoom_set(Evas_Object *obj, double zoom)
    int z;
    int zoom_changed = 0, started = 0;
    Ecore_Animator *an;
-   
+   if (!wd) return;
    if (zoom <= (1.0 / 256.0)) zoom = (1.0 / 256.0);
    if (zoom == wd->zoom) return;
    wd->zoom = zoom;
@@ -1306,9 +1360,11 @@ elm_photocam_zoom_set(Evas_Object *obj, double zoom)
  * @ingroup Photocam
  */
 EAPI double
-elm_photocam_zoom_get(Evas_Object *obj)
+elm_photocam_zoom_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) 1.0;
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return 1.0;
    return wd->zoom;
 }
 
@@ -1333,7 +1389,9 @@ elm_photocam_zoom_get(Evas_Object *obj)
 EAPI void
 elm_photocam_zoom_mode_set(Evas_Object *obj, Elm_Photocam_Zoom_Mode mode)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if (wd->mode == mode) return;
    wd->mode = mode;
      {
@@ -1354,9 +1412,11 @@ elm_photocam_zoom_mode_set(Evas_Object *obj, Elm_Photocam_Zoom_Mode mode)
  * @ingroup Photocam
  */
 EAPI Elm_Photocam_Zoom_Mode
-elm_photocam_zoom_mode_get(Evas_Object *obj)
+elm_photocam_zoom_mode_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) ELM_PHOTOCAM_ZOOM_MODE_LAST;
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return ELM_PHOTOCAM_ZOOM_MODE_LAST;
    return wd->mode;
 }
 
@@ -1374,9 +1434,11 @@ elm_photocam_zoom_mode_get(Evas_Object *obj)
  * @ingroup Photocam
  */
 EAPI void
-elm_photocam_image_size_get(Evas_Object *obj, int *w, int *h)
+elm_photocam_image_size_get(const Evas_Object *obj, int *w, int *h)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if (w) *w = wd->size.imw;
    if (h) *h = wd->size.imh;
 }
@@ -1388,12 +1450,12 @@ elm_photocam_image_size_get(Evas_Object *obj, int *w, int *h)
  * 
  */
 EAPI void
-elm_photocam_region_get(Evas_Object *obj, int *x, int *y, int *w, int *h)
+elm_photocam_region_get(const Evas_Object *obj, int *x, int *y, int *w, int *h)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord sx, sy, sw, sh;
-   int iw, ih;
-
+   if (!wd) return;
    elm_smart_scroller_child_pos_get(wd->scr, &sx, &sy);
    elm_smart_scroller_child_viewport_size_get(wd->scr, &sw, &sh);
    if (wd->size.w > 0)
@@ -1453,11 +1515,12 @@ elm_photocam_region_get(Evas_Object *obj, int *x, int *y, int *w, int *h)
  * @ingroup Photocam
  */
 EAPI void
-elm_photocam_image_region_show(Evas_Object *obj, int x, int y, int w, int h)
+elm_photocam_image_region_show(Evas_Object *obj, int x, int y, int w, int h __UNUSED__)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    int rx, ry, rw, rh;
-   
+   if (!wd) return;
    if ((wd->size.imw < 1) || (wd->size.imh < 1)) return;
    rx = (x * wd->size.w) / wd->size.imw;
    ry = (y * wd->size.h) / wd->size.imh;
@@ -1492,11 +1555,12 @@ elm_photocam_image_region_show(Evas_Object *obj, int x, int y, int w, int h)
  * @ingroup Photocam
  */
 EAPI void
-elm_photocam_image_region_bring_in(Evas_Object *obj, int x, int y, int w, int h)
+elm_photocam_image_region_bring_in(Evas_Object *obj, int x, int y, int w, int h __UNUSED__)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    int rx, ry, rw, rh;
-   
+   if (!wd) return;
    if ((wd->size.imw < 1) || (wd->size.imh < 1)) return;
    rx = (x * wd->size.w) / wd->size.imw;
    ry = (y * wd->size.h) / wd->size.imh;
@@ -1509,7 +1573,7 @@ elm_photocam_image_region_bring_in(Evas_Object *obj, int x, int y, int w, int h)
    if (wd->zoom_animator)
      {
         wd->nosmooth--;
-        if (wd->nosmooth == 0) _smooth_update(obj);
+        if (!wd->nosmooth) _smooth_update(obj);
         ecore_animator_del(wd->zoom_animator);
         wd->zoom_animator = NULL;
         zoom_do(obj, 1.0);
@@ -1527,11 +1591,15 @@ elm_photocam_image_region_bring_in(Evas_Object *obj, int x, int y, int w, int h)
  * 
  * @param obj The photocam object
  * @param paused The pause state to set
+ *
+ * @ingroup Photocam
  */
 EAPI void
 elm_photocam_paused_set(Evas_Object *obj, Eina_Bool paused)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if (wd->paused == !!paused) return;
    wd->paused = paused;
    if (wd->paused)
@@ -1553,10 +1621,71 @@ elm_photocam_paused_set(Evas_Object *obj, Eina_Bool paused)
  * 
  * @param obj The photocam object
  * @return The current paused state
+ *
+ * @ingroup Photocam
  */
 EAPI Eina_Bool
-elm_photocam_paused_get(Evas_Object *obj)
+elm_photocam_paused_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
    return wd->paused;
 }
+
+/**
+ * Get the internal low-res image used for photocam
+ * 
+ * This gets the internal image object inside photocam. Do not modify it. It
+ * is for inspection only, and hooking callbacks to. Nothing else. It may be
+ * deleted at any time as well.
+ *
+ * @param obj The photocam object
+ * @return The internal image object handle, or NULL if none exists
+ *
+ * @ingroup Photocam
+ */
+EAPI Evas_Object *
+elm_photocam_internal_image_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
+   return wd->img;
+}
+
+/**
+ * Set the photocam scrolling bouncing.
+ *
+ * @param obj The photocam object
+ * @param h_bounce bouncing for horizontal
+ * @param v_bounce bouncing for vertical
+ * @ingroup Photocam
+ */
+EAPI void
+elm_photocam_bounce_set(Evas_Object *obj, Eina_Bool h_bounce, Eina_Bool v_bounce)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   elm_smart_scroller_bounce_allow_set(wd->scr, h_bounce, v_bounce);
+}
+
+
+/**
+ * Get the photocam scrolling bouncing.
+ *
+ * @param obj The photocam object
+ * @param h_bounce bouncing for horizontal
+ * @param v_bounce bouncing for vertical
+ * @ingroup Photocam
+ */
+EAPI void
+elm_photocam_bounce_get(const Evas_Object *obj, Eina_Bool *h_bounce, Eina_Bool *v_bounce)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   elm_smart_scroller_bounce_allow_get(wd->scr, h_bounce, v_bounce);
+}
+
