@@ -4,15 +4,13 @@
 static void
 _smart_extents_calculate(Evas_Object *box, Evas_Object_Box_Data *priv, int horizontal, int homogeneous)
 {
-   Evas_Coord minw, minh, maxw, maxh, mnw, mnh;
+   Evas_Coord minw, minh, mnw, mnh;
    const Eina_List *l;
    Evas_Object_Box_Option *opt;
 
    /* FIXME: need to calc max */
    minw = 0;
    minh = 0;
-   maxw = -1;
-   maxh = -1;
    if (homogeneous)
      {
 	EINA_LIST_FOREACH(priv->children, l, opt)
@@ -88,7 +86,7 @@ _els_box_layout(Evas_Object *o, Evas_Object_Box_Data *priv, int horizontal, int
 	     if (wy > 0.0) expand++;
 	  }
      }
-   if (expand == 0)
+   if (!expand)
      {
 	evas_object_size_hint_align_get(o, &ax, &ay);
 	if (horizontal)
